// 
// sussen-agent - Webservice.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Net;
using System.Reflection;
using System.Text;
using System.Web.Services.Description;
using System.Web.Services.Discovery;
using System.Xml;
using System.Xml.Schema;
using Microsoft.CSharp;
using ICSharpCode.SharpZipLib.Checksums;
using ICSharpCode.SharpZipLib.Zip;
using Sussen.Oval;

namespace Sussen.Agent.Webservice
{
	public class WebMethods
	{

		public static string GetDefinitions ()
		{
			try {
			
                ServiceConfig sc = new ServiceConfig (ConfigurationManager.AppSettings.Get("webserviceWSDL"));

				Assembly newAssembly = sc.GetAssembly ();
			
				// assumes first class is always the one that we want	
				Type[] allTypes = newAssembly.GetTypes ();			
				Type t = newAssembly.GetType (allTypes[0].ToString());
				object obj = Activator.CreateInstance (t);

				string systemInfo = Interpreter.GetSystemInfo ();
				byte[] siBytes = Encoding.UTF8.GetBytes (systemInfo);
				
#if MONO_BUILD				
				string familyName = "unix";
#else
				string familyName = "windows";
#endif				
			
				object[] methodParams = new object[] { "zip", familyName, siBytes, null };
				ParameterModifier p = new ParameterModifier(4);
				p[0] = false;
				p[1] = false;
				p[2] = false;
				p[3] = true;				

				ParameterModifier[] mods = { p };
						
				int result = (int) obj.GetType().InvokeMember("GetDefinitions", BindingFlags.InvokeMethod, null, obj, methodParams, mods, null, null);
				Log.Error ("GetDefinitions result {0}", result);
				CheckReturnCode ("GetDefinitions", result);

				byte[] defBytes = (byte[]) methodParams[3];
       			string defXml = DecodeAndUnzipFile (defBytes);
       			return defXml;

			} catch (Exception e) {
			
				Console.WriteLine (I18N.ErrorGeneric, e.Message);		
				Log.Debug (e);				
			}
			
			return null;
		}

		public static void RegisterAgent (string serviceUri)
		{
			if (serviceUri == null)
				throw new ArgumentNullException ("web service URI");

			if (File.Exists ("sussen-agent.id")) {
				Console.Write (I18N.QuestionReRegisterAgent);
				string answer = Console.ReadLine ();
				answer = answer.ToLower ();

				if (answer == "n" || answer == "no") {
					Console.WriteLine (I18N.ErrorAbort);
					return;
				}
			}
			
			try  {

				string systemInfo = Interpreter.GetSystemInfo ();
				byte[] siBytes = Encoding.UTF8.GetBytes (systemInfo);

				ServiceConfig sc = new ServiceConfig (serviceUri);			
				Assembly newAssembly = sc.GetAssembly ();

				// assumes first class is always the one that we want	
				Type[] allTypes = newAssembly.GetTypes ();			
				Type t = newAssembly.GetType (allTypes[0].ToString());

				object obj = Activator.CreateInstance (t);

				object[] methodParams = new object[] { "none", siBytes, null };
				ParameterModifier p = new ParameterModifier(3);
				p[0] = false;
				p[1] = false;
				p[2] = true;

				ParameterModifier[] mods = { p };
						
				int result = (int) obj.GetType().InvokeMember("RequestClientID", BindingFlags.InvokeMethod, null, obj, methodParams, mods, null, null);				

				Log.Debug ("RequestClientID result: {0}", result);				
				CheckReturnCode ("RequestClientID", result);

				byte[] rawBytes = (byte[]) methodParams[2];						
				string res = Encoding.UTF8.GetString (rawBytes);
		
				string agentID = GetHostIdentification (res);		
				if (agentID == null)
					throw new ApplicationException (I18N.ErrorUnableToRegisterAgent);

				Assembly asm = Assembly.GetExecutingAssembly ();
				string asm_dir = System.IO.Path.GetDirectoryName (asm.Location);
				string agentIDFile = Path.Combine (asm_dir, "sussen-agent.id");

				StreamWriter sw = new StreamWriter(agentIDFile);
				sw.WriteLine (agentID);
				sw.Close ();

				Console.WriteLine (I18N.AgentRegistered, Defines.PACKAGE, serviceUri);
				
			} catch (Exception e) {
			
				Console.WriteLine (I18N.ErrorGeneric, e.Message);
			}
		}

		public static void SendResults (string resultData)
		{
			if (resultData == null)
				throw new ArgumentNullException ("result data");

			try {

                ServiceConfig sc = new ServiceConfig(ConfigurationManager.AppSettings.Get("webserviceWSDL"));

				Assembly newAssembly = sc.GetAssembly ();
			
				// assumes first class is always the one that we want	
				Type[] allTypes = newAssembly.GetTypes ();			
				Type t = newAssembly.GetType (allTypes[0].ToString());
				object obj = Activator.CreateInstance (t);

				byte[] encodedData = ZipAndEncodeFile  (resultData);

				List<object> methodParams = new List<object>();
				methodParams.Add ("zip");
				methodParams.Add (null);
				methodParams.Add (encodedData);
				
				MethodInfo mi = obj.GetType().GetMethod ("ReturnResults");			
				int result = (int) mi.Invoke (obj, methodParams.ToArray());

				Log.Debug ("SendResults result: {0}", result);				
				CheckReturnCode ("ReturnResults", result);

				Log.Debug (I18N.ScanDataSaved);

			} catch (Exception e) {
			
				Console.WriteLine (I18N.ErrorGeneric, e.Message);
				Log.Debug (e);				
			}
		}
		
		public static void SendSystemCharacteristics (string scData)
		{
			if (scData == null)
				throw new ArgumentNullException ("scData");

			try {

                ServiceConfig sc = new ServiceConfig(ConfigurationManager.AppSettings.Get("webserviceWSDL"));

				Assembly newAssembly = sc.GetAssembly ();
			
				// assumes first class is always the one that we want	
				Type[] allTypes = newAssembly.GetTypes ();			
				Type t = newAssembly.GetType (allTypes[0].ToString());
				object obj = Activator.CreateInstance (t);

				byte[] encodedData = ZipAndEncodeFile  (scData);

				List<object> methodParams = new List<object>();
				methodParams.Add ("zip");
				methodParams.Add (null);
				methodParams.Add (encodedData);
				
				MethodInfo mi = obj.GetType().GetMethod ("ReturnSystemCharacteristics");			
				int result = (int) mi.Invoke (obj, methodParams.ToArray());
				
				Log.Debug ("SendSystemCharacteristics result: {0}", result);
 				CheckReturnCode ("SendSystemCharacteristics", result);
 				
				Log.Debug (I18N.SystemCharacteristicsSaved);

			} catch (Exception e) {
			
				Console.WriteLine (I18N.ErrorGeneric, e.Message);
				Log.Debug (e);				
			}
		}
		
		#region internal_methods
		
		static void CheckReturnCode (string methodName, int returnCode)
		{
			switch (returnCode) {
			
			case 0:
				break;

			// Function is stubbed				
			case 1:
				break;

			case 2:
				throw new Exception (methodName + I18N.ErrorWebserviceFailedToMatchSchemaSomeProcessing);

			// Success; no data changed
			case 3:
				break;
				
			case -1:
				throw new Exception (methodName + I18N.ErrorWebserviceInvalidXml);
				
			case -2:
				throw new Exception (methodName + I18N.ErrorWebserviceFailedToMatchSchemaNoProcessing);
				
			case -3:
				throw new Exception (methodName + I18N.ErrorWebserviceDecompression);

			case -4:
				throw new Exception (methodName + I18N.ErrorWebserviceFatalError);
			
			default:
				break;
			}			
		}
		
		static string GetHostIdentification (string xml)
		{
	    	try {
	    		
	    		XmlDocument doc = new XmlDocument ();
		    	doc.LoadXml (xml);
		    	
		    	XmlNodeList nl = doc.GetElementsByTagName ("host_identification");
		    	if (!(nl.Count > 0))
		    		return null;
		    		
		    	XmlElement xe = (XmlElement) nl[0];
		    	return xe.InnerText;
		    	
		    } catch {	    
		    	return null;
		    }
		}

		static byte[] ZipAndEncodeFile (string resultData)
		{
			byte[] buf = Encoding.UTF8.GetBytes (resultData);

			MemoryStream ms = new MemoryStream(); 
			ZipOutputStream zs = new ZipOutputStream(ms);
			zs.SetLevel (6);

			Crc32 crc = new Crc32();

			ZipEntry ze = new ZipEntry ("results.xml");
			ze.DateTime = DateTime.Now;
			ze.Size = buf.Length;

			crc.Reset ();
			crc.Update(buf);

			ze.Crc = crc.Value;

			zs.PutNextEntry (ze);
			zs.Write(buf, 0, buf.Length);
			zs.Finish ();

			byte[] zipBytes = new byte [ms.Length];
			ms.Seek (0, SeekOrigin.Begin);
			ms.Read (zipBytes, 0, zipBytes.Length);

			zs.Close ();
			ms.Close ();

			return zipBytes;
		}

		static string DecodeAndUnzipFile(byte[] encodedFile)
	    {
	        MemoryStream ms = new MemoryStream();
	        ms.Write(encodedFile, 0, encodedFile.Length);
	        ms.Seek(0, SeekOrigin.Begin);

	        ZipInputStream zs = new ZipInputStream(ms);
	        MemoryStream outMemoryStream = new MemoryStream();

	        while (zs.GetNextEntry() != null)
	        {
	            int size = 4096;
	            byte[] data = new byte[4096];

	            do
	            {
	                size = zs.Read(data, 0, data.Length);
	                outMemoryStream.Write(data, 0, size);
	            } while (size > 0);
	        }

	        zs.Close();
	        ms.Close();

			return Encoding.UTF8.GetString(outMemoryStream.ToArray());
	    }
	    
	    #endregion
	}

	public class AssemblyGenerator
	{
		public AssemblyGenerator ()
		{
		}
		
		public Assembly Build (ArrayList descriptions, ArrayList schemas)
		{
			CodeNamespace codeNamespace = new CodeNamespace ("Sussen.Agent.WebService");

			ServiceDescriptionImporter importer = new ServiceDescriptionImporter();
			importer.ProtocolName = "Soap";
			
			foreach (ServiceDescription sd in descriptions)
				importer.AddServiceDescription(sd, null, null);
					
			foreach (XmlSchema sc in schemas)
				importer.Schemas.Add (sc);
				
			importer.Import(codeNamespace, null);

			CodeDomProvider provider = new CSharpCodeProvider();

			StringBuilder sb = new StringBuilder();
			StringWriter sw = new StringWriter (sb);

            provider.GenerateCodeFromNamespace(codeNamespace, sw, null);
		
			string src = sb.ToString();
			sw.Close();

			CompilerParameters cp = new CompilerParameters();
			cp.ReferencedAssemblies.Add("System.dll");
            cp.ReferencedAssemblies.Add("System.Xml.dll");
			cp.ReferencedAssemblies.Add("System.Web.Services.dll");
			cp.ReferencedAssemblies.Add(System.Reflection.Assembly.GetExecutingAssembly().Location);

            CompilerResults cr = provider.CompileAssemblyFromSource(cp, src);

            if (cr.Errors.Count > 0)
            {
                foreach (CompilerError ce in cr.Errors)
                    Console.WriteLine(ce.ToString());

                throw new Exception(string.Format(I18N.ErrorWebserviceBuildFailed, cr.Errors.Count));
            }

			return cr.CompiledAssembly; 				
		}
	}

	public class ServiceConfig
	{
		AssemblyGenerator generator;

		ArrayList descriptions;
		ArrayList schemas;

		string wsdlLocation;

		public ServiceConfig (string wsdlUri)
		{
			wsdlLocation = wsdlUri;
			
			generator = new AssemblyGenerator ();

			descriptions = new ArrayList ();
			schemas = new ArrayList ();
		}
		
		public Assembly GetAssembly ()
		{
			DiscoveryClientProtocol dcc = new DiscoveryClientProtocol ();

            string domain = ConfigurationManager.AppSettings.Get("webserviceDomain");
            string user = ConfigurationManager.AppSettings.Get("webserviceUser");
            string pass = ConfigurationManager.AppSettings.Get("webservicePass");

            if (domain != "" || domain != null || user != "" || user != null || pass != "" || pass != null) {
				
				NetworkCredential credentials = new NetworkCredential();
				
				if (user != "" || user != null)
					credentials.UserName = user;
					
				if (pass != "" || pass != null)
					credentials.Password = pass;
					
				if (domain != "" || domain != null)
					credentials.Domain = domain;
					
				dcc.Credentials = credentials;
			}

			dcc.AllowAutoRedirect = true;			
			dcc.DiscoverAny (wsdlLocation);
			dcc.ResolveAll ();

			foreach (object doc in dcc.Documents.Values) {
			
				if (doc is ServiceDescription)
					descriptions.Add ((ServiceDescription)doc);
				else if (doc is XmlSchema)
					schemas.Add ((XmlSchema)doc);
			}
			
			if (descriptions.Count == 0)
				throw new Exception (I18N.ErrorNoWSDL + wsdlLocation);
						
			Assembly asm = generator.Build (descriptions, schemas);
			
			return asm;
		}
	}
}
