// 
// sussen-applet - Applet.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Runtime.InteropServices;
using System.Text;
using Gtk;
using Gnome;
using Mono.Unix;

namespace Sussen.Applet
{
	public class SussenPanelApplet : Gnome.PanelApplet
	{	
		static Gnome.Program program;	
		static Gdk.Pixbuf panelIcon;
		static Gdk.Pixbuf sussenLogo;

		protected SussenPanelApplet (IntPtr raw) : base (raw)
		{
			panelIcon = new Gdk.Pixbuf (null, "sussen-icon.png"); 
			sussenLogo = new Gdk.Pixbuf (null, "sussen.png");
		}

		static void PrintUsage ()
		{
			StringBuilder usage = new StringBuilder ();
			usage.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			usage.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			usage.AppendLine ();
			usage.AppendFormat ("{0} [OPTIONS]\n\n", Defines.PACKAGE);
			usage.AppendLine ("Options:");
			usage.AppendLine ("  --help\t\tPrint this usage message.");
			usage.AppendLine ("  --panel-applet\tRun as GNOME panel applet.");			
			usage.AppendLine ("  --version\t\tPrint version and exit.");
						
			Console.WriteLine (usage.ToString());
		}

		static void PrintVersion ()
		{
			StringBuilder version = new StringBuilder ();
			version.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			version.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			version.AppendLine ("This is free software; see the source for copying conditions. There is NO");
			version.AppendLine ("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.");
		
			Console.WriteLine (version.ToString());
		}

        [DllImport("libc")]
		static extern int prctl(int option, byte [] arg2, ulong arg3 , ulong arg4, ulong arg5);
        
		static void SetProcessName (string name)
	    {
	    	try {
				if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name), 0, 0, 0) != 0)
					Console.WriteLine ("Error setting process name: {0}", Mono.Unix.Native.Stdlib.GetLastError());
			} catch {}
		}

		static void Main (string[] args)
		{
			bool runAsPanelApplet = false;
		
			foreach (string arg in args) {
			
				switch (arg) {
				
				case "--panel-applet":
					runAsPanelApplet = true;
					break;
				
				case "--help":
					PrintUsage ();
					Environment.Exit (0);
					break;				
				
				case "--version":
					PrintVersion ();
					Environment.Exit (0);
					break;
				}
			}
		
			SetProcessName ("sussen-applet");
		
			program = new Gnome.Program (Defines.PACKAGE, Defines.VERSION, Gnome.Modules.UI, args);
						
			if (runAsPanelApplet)
				PanelAppletFactory.Register (typeof (SussenPanelApplet));
			else
				StartTrayIcon ();				
		}
		
		static void StartTrayIcon ()
		{
			SussenTrayIcon trayIcon = new SussenTrayIcon ();
			trayIcon.Show ();
			
			Console.WriteLine ("If you're wondering whether {0} is working check your system tray", Defines.PACKAGE);
			
			program.Run ();
		}
		
		public override string IID 
		{
			get { return "OAFIID:SussenApplet"; }
		}

		public override string FactoryIID 
		{
			get { return "OAFIID:SussenApplet_Factory"; }
		}

		public override void Creation ()
		{								
			Gtk.Image image = new Gtk.Image (panelIcon);
			this.Add (image);			
			this.ShowAll ();	
			
			BonoboUIVerb [] menuVerbs;
			menuVerbs = new BonoboUIVerb [] {
				new BonoboUIVerb ("Scan", new ContextMenuItemCallback (ShowScanSystemDialog)),
				new BonoboUIVerb ("Analyze", new ContextMenuItemCallback (ShowAnalyzeDataDialog)),
				new BonoboUIVerb ("About", new ContextMenuItemCallback (ShowAbout))
			};
			
			this.SetupMenu (Defines.MENU_XML, menuVerbs);

			Gtk.Tooltips tips = new Gtk.Tooltips ();
			tips.SetTip (this, "Security scanner", null);
			tips.Enable ();
			
			Console.WriteLine ("{0} created.", Defines.PACKAGE);
			
			program.Run ();
		}

		void ShowScanSystemDialog ()
		{	
			new AppletDialog (AppletDialogType.Scan);
		}

		void ShowAnalyzeDataDialog ()
		{
			new AppletDialog (AppletDialogType.Analyze);
		}
		
		void ShowAbout ()
		{
			string[] author = { "Loren Bandiera" };
			string[] docs = { "Loren Bandiera" };
			string translators = "";
			string desc = "Security scanner";
			string copy = "Copyright \xa9 2004-2006 MMG Security, Inc.";
			
			Gnome.About ab = new Gnome.About (
				Defines.PACKAGE,
				Defines.VERSION,
				copy,
				desc,
				author,
				docs, 
				translators, 
				sussenLogo);

			ab.Icon = panelIcon;
			ab.Show ();
		}
	}
}
