// 
// sussen-applet - TrayIcon.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Runtime.InteropServices;
using Gtk;

namespace Sussen.Applet
{
	public class SussenTray : Gtk.EventBox
	{
		Gtk.Tooltips tips;
		Gtk.Image image;
		
		public SussenTray ()
		{
			this.image = new Gtk.Image (new Gdk.Pixbuf (null, "sussen-icon.png"));

			this.CanFocus = true;
			this.Add (image);
			this.ShowAll ();
			
			string tipText = "Sussen security scanner";
			
			tips = new Gtk.Tooltips ();
			tips.SetTip (this, tipText, null);
			tips.Enable ();
		}
	}
	
	public class SussenTrayIcon : Gtk.Plug
	{
		[DllImport ("libsussenglue")]
		private static extern IntPtr egg_tray_icon_new (string name);
		
		private static SussenTray tray;
		private static Gdk.Pixbuf panelIcon;
		private static Gdk.Pixbuf sussenLogo;
				
		public SussenTrayIcon ()
		{
			this.Raw = egg_tray_icon_new (Defines.PACKAGE);
			
			panelIcon = new Gdk.Pixbuf (null, "sussen-icon.png"); 
			sussenLogo = new Gdk.Pixbuf (null, "sussen.png");
			
			tray = new SussenTray ();
			tray.ButtonPressEvent += ButtonPress;		
			this.Add (tray);
			
			this.ShowAll ();
		}
		
		public void ShowScanDialog ()
		{
			new AppletDialog (AppletDialogType.Scan);
		}
		
		void ButtonPress (object sender, Gtk.ButtonPressEventArgs args) 
		{
			Gtk.Widget parent = (Gtk.Widget) sender;

			if (args.Event.Button == 1) {
				
				ShowScanDialog ();
				
			} else if (args.Event.Button == 3) {
				
				Gtk.Menu menu = MakeRightClickMenu (parent);
				
				menu.Popup (null, null, null, 3,Gtk.Global.CurrentEventTime);

				args.RetVal = true;
			}
		}
		
		Gtk.Menu MakeRightClickMenu (Gtk.Widget parent)
		{
			Gtk.Menu menu = new Gtk.Menu ();
			menu.AttachToWidget (parent, null);

			Gtk.AccelGroup accel_group = new Gtk.AccelGroup ();
			menu.AccelGroup = accel_group;

			Gtk.ImageMenuItem item;

			item = new Gtk.ImageMenuItem ("S_can system...");
			item.Image = new Gtk.Image (Gtk.Stock.Execute, Gtk.IconSize.Menu);
			item.Activated += ScanSystem;
			menu.Append (item);

			item = new Gtk.ImageMenuItem ("_Analyze existing data...");
			item.Image = new Gtk.Image (Gtk.Stock.Find, Gtk.IconSize.Menu);
			item.Activated += AnalyzeData;
			menu.Append (item);

			item = new Gtk.ImageMenuItem ("_About");
			item.Image = new Gtk.Image (Gnome.Stock.About, Gtk.IconSize.Menu);
			item.Activated += ShowAbout;
			menu.Append (item);

			menu.Append (new Gtk.SeparatorMenuItem ());

			item = new Gtk.ImageMenuItem ("_Quit");
			item.Image = new Gtk.Image (Gtk.Stock.Quit, Gtk.IconSize.Menu);
			item.Activated += Quit;
			menu.Append (item);

			menu.ShowAll ();
			return menu;
		}
		
		void ScanSystem (object sender, EventArgs args)
		{
			ShowScanDialog ();
		}
		
		void AnalyzeData (object sender, EventArgs args)
		{
			new AppletDialog (AppletDialogType.Analyze);
		}
		
		void ShowAbout (object sender, EventArgs args)
		{
			string[] author = { "Loren Bandiera" };
			string[] docs = { "Loren Bandiera" };
			string translators = "";
			string desc = "Security scanner";
			string copy = "Copyright \xa9 2005 MMG Security, Inc.";
			
			Gnome.About ab = new Gnome.About (
				Defines.PACKAGE,
				Defines.VERSION,
				copy,
				desc,
				author,
				docs, 
				translators, 
				sussenLogo);

			ab.Icon = panelIcon;
			ab.Show ();		
		}
		
		void Quit (object sender, EventArgs args)
		{
			Environment.Exit (0);
		}
	}
}