// 
// sussen-editor - DefinitionView.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Xml;
using Gtk;
using Sussen.Oval;

namespace Sussen.Editor.Gui
{
	public class DefinitionEditedEventArgs : EventArgs
	{
		public string ID;
	}

	public delegate void DefinitionEditedHandler (object o, DefinitionEditedEventArgs args);

	public class DefinitionView : ScrolledWindow
	{
		Entry idEntry;
		Entry titleEntry;
		Entry familyEntry;
		Entry platformEntry;
		Entry productEntry;
		Entry refSourceEntry;
		Entry refIDEntry;
		Entry refURLEntry;
		TextView descTextView;

		TreeView critTreeView;
		ListStore critStore;

		ComboBox critOperatorComboBox;

		DefinitionElement currentDefinition;
		TestElement[] currentTests;
		bool firstTime;
	
		public event DefinitionEditedHandler DefinitionEdited;
	
		public DefinitionView() : base ()
		{
			VBox rv = new VBox ();
		
			HBox h = new HBox ();
			h.BorderWidth = 6;
			h.Spacing = 6;
			rv.PackStart (h, false, false, 0);
			
			VBox v = new VBox ();
			v.Spacing = 6;
			h.PackStart (v, false, false, 0);
	
			Label l = new Label ("ID:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);
			
			l = new Label ("Title:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Family:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Platform:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Product:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Reference source:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Reference ID:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Reference URL:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Description:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);
			
			v = new VBox ();
			v.Spacing = 6;
			h.PackStart (v, true, true, 0);

			idEntry = new Entry();
			idEntry.Changed += OnIdChanged;
			
			titleEntry = new Entry();
			titleEntry.Changed += OnTitleChanged;
			
			familyEntry = new Entry();
			familyEntry.Changed += OnFamilyChanged;
			
			platformEntry = new Entry();
			platformEntry.Changed += OnPlatformChanged;
			
			productEntry = new Entry();
			productEntry.Changed += OnProductChanged;
			
			refSourceEntry = new Entry();
			refSourceEntry.Changed += OnRefSourceChanged;
			
			refIDEntry = new Entry();
			refIDEntry.Changed += OnRefIdChanged;
			
			refURLEntry = new Entry();
			refURLEntry.Changed += OnRefUrlChanged;
			
			descTextView = new TextView ();
			descTextView.WrapMode = WrapMode.Word;
			descTextView.Buffer.Changed += OnDescriptionChanged;
			
			v.PackStart (idEntry, true, true, 0);
			v.PackStart (titleEntry, true, true, 0);
			v.PackStart (familyEntry, true, true, 0);
			v.PackStart (platformEntry, true, true, 0);
			v.PackStart (productEntry, true, true, 0);
			v.PackStart (refSourceEntry, true, true, 0);
			v.PackStart (refIDEntry, true, true, 0);
			v.PackStart (refURLEntry, true, true, 0);
			v.PackStart (descTextView, true, true, 0);
			
			VBox v2 = new VBox ();
			rv.PackEnd (v2, true, true, 0);
			
			HBox h2 = new HBox ();
			v2.PackStart (h2, false, false, 5);
			l = new Label ();
			l.UseMarkup = true;
			l.Markup = "<b>Criteria</b>";
			h2.PackStart (l, false, false, 5);

			h2 = new HBox ();
			v2.PackStart (h2, false, false, 5);
			l = new Label ("Operator");
			h2.PackStart (l, false, false, 5);
			
			critOperatorComboBox = ComboBox.NewText ();
			critOperatorComboBox.AppendText ("AND");
			critOperatorComboBox.AppendText ("OR");
			critOperatorComboBox.AppendText ("XOR");
			critOperatorComboBox.Active = 0;
			critOperatorComboBox.Changed += OnCriteriaOperatorChanged;
			critOperatorComboBox.Show ();			
			h2.PackStart (critOperatorComboBox, false, false, 5);
			
			h2 = new HBox ();
			v2.PackStart (h2, true, true, 5);
			
			ScrolledWindow sw = new ScrolledWindow ();
			sw.Show ();
			
			critStore = new ListStore (typeof (string), typeof (string));
			critTreeView = new TreeView ();
			critTreeView.Model = critStore;
			critTreeView.AppendColumn ("Test", new CellRendererText (), "text", 0);
			critTreeView.AppendColumn ("Comment", new CellRendererText (), "text", 1);

			sw.AddWithViewport (critTreeView);
			h2.PackStart (sw, true, true, 5);

			h2 = new HBox ();
			v2.PackStart (h2, false, false, 5);
			
			Button b1 = new Button (Stock.Add);
			b1.Clicked += OnAddCriteria;
			h2.PackStart (b1, false, false, 5);
			
			Button b2 = new Button (Stock.Remove);
			b2.Clicked += OnRemoveCriteria;
			h2.PackStart (b2, false, false, 5);
			
			this.AddWithViewport (rv);
			this.ShowAll ();
		}
		
		public DefinitionElement CurrentDefinition
		{
			set {
				currentDefinition = value;
				firstTime = true;
				ShowDefinition ();
			}
		}
		
		public TestElement[] CurrentTests
		{
			set { currentTests = value; } 
		}
		
		string GetCriteriaOperator ()
		{
			TreeIter iter;
			critOperatorComboBox.GetActiveIter (out iter);

			string op = (string) critOperatorComboBox.Model.GetValue (iter, 0);
			return op;
		}
		
		void SetCriteriaOperation (string op) 
		{
			switch (op.ToUpper()) {
			case "AND":
				critOperatorComboBox.Active = 0;
				break;
			
			case "OR":
				critOperatorComboBox.Active = 1;
				break;
						
			case "XOR":
				critOperatorComboBox.Active = 2;
				break;
			
			default:
				break;
			}
		}
		
		void DispatchDefinitionEditedEvent ()
		{		
			if (DefinitionEdited != null) {
				DefinitionEditedEventArgs dargs = new DefinitionEditedEventArgs ();
				dargs.ID = currentDefinition.ID;
				
				DefinitionEdited (this, dargs);
			}
		}
		
		void OnIdChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.ID = idEntry.Text;
				DispatchDefinitionEditedEvent ();
			}
		}
		
		void OnTitleChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("title", titleEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}
		
		void OnFamilyChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("affected_family", familyEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}

		void OnPlatformChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("affected_platform", platformEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}
		
		void OnProductChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("affected_product", productEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}

		void OnRefSourceChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("reference_source", refSourceEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}

		void OnRefIdChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("reference_id", refIDEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}
		
		void OnRefUrlChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("reference_url", refURLEntry.Text);
				DispatchDefinitionEditedEvent ();
			}
		}		
		
		void OnDescriptionChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				currentDefinition.SetMetaData ("description", descTextView.Buffer.Text);
				DispatchDefinitionEditedEvent ();
			}		
		}
		
		void OnCriteriaOperatorChanged (object o, EventArgs args)
		{
			if (currentDefinition != null && !firstTime) {
				CriteriaElement ce = currentDefinition.Criteria[0];
				ce.Operator = GetCriteriaOperator ();
				DispatchDefinitionEditedEvent ();
			}		
		}
		
		void OnAddCriteria (object o, EventArgs args)
		{
			GtkAddTestDialog dlg = new GtkAddTestDialog ();
			foreach (TestElement te in currentTests)
				dlg.AddTest (te.ID, te.Comment);
				
			ResponseType response = (ResponseType) dlg.Run ();
			if (response == ResponseType.Ok) {
				foreach (string k in dlg.EnabledTests.AllKeys) {				
					string v = dlg.EnabledTests[k];
					Sussen.Oval.Log.Debug ("Adding criteria {0} ({1}) to {2}", k, v, currentDefinition.ID);
				
					critStore.AppendValues (k, v);					
					CriteriaElement ce = currentDefinition.Criteria[0];
					ce.Criterions.Add (k, v);
				}
			}
			
			dlg.Destroy ();
		}
		
		void OnRemoveCriteria (object o, EventArgs args)
		{
			TreeModel model;
			TreeIter iter;

			if (critTreeView.Selection.GetSelected (out model, out iter)) {
	
				string name = (string) critStore.GetValue (iter, 0);
				Sussen.Oval.Log.Debug ("Removing criteria {0} from {1}", name, currentDefinition.ID);
				
				critStore.Remove (ref iter);
				
				CriteriaElement ce = currentDefinition.Criteria[0];
				ce.Criterions.Remove (name);
			
				DispatchDefinitionEditedEvent ();
			}		
		}
		
		void ShowDefinition ()
		{
			if (currentDefinition == null)
				return;
		
			idEntry.Text = currentDefinition.ID;
			titleEntry.Text = currentDefinition.GetMetaData ("title");
			familyEntry.Text = currentDefinition.GetMetaData ("affected_family");
			platformEntry.Text = currentDefinition.GetMetaData ("affected_platform");
			productEntry.Text = currentDefinition.GetMetaData ("affected_product");
			refSourceEntry.Text = currentDefinition.GetMetaData ("reference_source");
			refIDEntry.Text = currentDefinition.GetMetaData ("reference_id");
			refURLEntry.Text = currentDefinition.GetMetaData ("reference_url");
			descTextView.Buffer.Text = currentDefinition.GetMetaData ("description");
				
			critStore.Clear ();
			if (currentDefinition.Criteria.Length > 0) {
				CriteriaElement ce = currentDefinition.Criteria[0];
				SetCriteriaOperation (ce.Operator);
				foreach (string k in ce.Criterions.Keys) {
					string v = ce.Criterions[k];
					critStore.AppendValues (k, v);
				}					
			}
			
			if (firstTime)
				firstTime = false;
		}
	}
}
