// 
// sussen-editor - GtkFileView.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Xml;
using Gtk;
using GLib;
using Sussen.Oval;

namespace Sussen.Editor.Gui
{
	public class GtkFileView : Gtk.Bin
	{
		protected TreeStore fileStore;
		protected TreeIter rootIter;
		protected TreeIter defIter;
		protected TreeIter testIter;
		protected string fileName;
		
		protected Gtk.TreeView fileTreeView;

		
		public GtkFileView()
		{
			Stetic.Gui.Build(this, typeof(Sussen.Editor.Gui.GtkFileView));
			
			fileStore = new TreeStore (typeof (string));
			fileTreeView.Model = fileStore;
			fileTreeView.AppendColumn ("fileRoot", new CellRendererText (), "text", 0);
			
			// FIXME: set in designer but it had no effect
			fileTreeView.RowActivated += OnRowActivated;
			fileTreeView.ButtonPressEvent += OnRightClick;
		}
		
		public void SetFileDirty (bool state)
		{
			string name = (string) fileStore.GetValue (rootIter, 0);
			
			if (state)
				name = string.Format ("*{0}", name);
			else
				name = name.Remove (0, 1);
				
			fileStore.SetValue (rootIter, 0, name);
		}
		
		public void SetOpenFile (string fileName, DefinitionsDocument doc)
		{
			this.fileName = fileName;
			
			fileStore.Clear ();		
			
			rootIter = fileStore.AppendValues (fileName);
			
			defIter = fileStore.AppendValues (rootIter, "Definitions");
			foreach (string k in doc.Definitions.Keys)
				fileStore.AppendValues (defIter, k);
				
			testIter = fileStore.AppendValues (rootIter, "Tests");
			foreach (string k in doc.Tests.Keys)
				fileStore.AppendValues (testIter, k);
			
			fileTreeView.ExpandAll ();			
		}
		
		public string Filename
		{
			set {
				fileStore.SetValue (rootIter, 0, value);
			}
		}
		
		public string Selected
		{
			get {
				TreeModel model;
				TreeIter iter;
				string name = null;

				if (fileTreeView.Selection.GetSelected (out model, out iter)) {
					name = (string) fileStore.GetValue (iter, 0);
					if (name == "Definitions" || name == "Tests" || name == this.fileName)
						name = null;
				}

				return name;			
			}
		}
		
		public event DefinitionCreatedHandler DefinitionCreated;		
		public event ItemSelectedHandler ItemSelected;
		public event TestCreatedHandler TestCreated;
		
		#region internal_methods
		
		void DoPopUp()
		{
			Menu popup = new Menu();
			
			MenuItem defItem = new MenuItem ("New definition");
			defItem.Activated += new EventHandler (OnNewDefinitionActivate);
			defItem.Show ();
			popup.Append (defItem);	

			MenuItem testItem = new MenuItem ("New test");
			testItem.Show ();			
			
			Menu typeMenu = new Menu ();
			
			foreach (string s in Defines.SupportedTests) {
				MenuItem mi = new MenuItem (Vocabulary.GetTestName (s));
				mi.Activated += new EventHandler (OnNewTestActivate);
				mi.Show ();
				typeMenu.Append (mi);			
			}			
			
			testItem.Submenu = typeMenu;
			popup.Append (testItem);
			
			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}
		
		#endregion
		
		void OnNewDefinitionActivate (object o, EventArgs args)
		{
			DefinitionCreatedEventArgs da = new DefinitionCreatedEventArgs ();
			
			if (DefinitionCreated != null) {
				DefinitionCreated (this, da);
				fileStore.AppendValues (defIter, da.Name);
			}
		}

		void OnNewTestActivate (object o, EventArgs args)
		{
			MenuItem mi = (MenuItem) o;
			Label l = (Label) mi.Child;
			
			TestCreatedEventArgs targs = new TestCreatedEventArgs ();
			targs.Type = Vocabulary.GetTestType (l.Text);
			
			if (TestCreated != null) {
				TestCreated (this, targs);				
				fileStore.AppendValues (testIter, targs.Name);
			}
		}

		[ConnectBefore]
		protected virtual void OnRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp ();
		}

		protected virtual void OnRowActivated (object o, RowActivatedArgs args)
		{
			TreePath path = args.Path;
			TreeIter iter;
			
			if (fileStore.GetIter (out iter, path)) {

				string name = null;
				name = (string) fileStore.GetValue (iter, 0);
				
				ItemSelectedEventArgs itemArgs = new ItemSelectedEventArgs ();
				itemArgs.Name = name;
				
				if (fileStore.IsAncestor (defIter, iter))
					itemArgs.IsDefinition = true;
				else
					itemArgs.IsDefinition = false;
								
				if (ItemSelected != null)
					ItemSelected (this, itemArgs);				
			}
		}		
	}
	
	public class ItemSelectedEventArgs : EventArgs
	{
		public string Name;
		public bool IsDefinition;
	}
	
	public class TestCreatedEventArgs : EventArgs
	{
		public string Name;
		public string Type;
	}
	
	public class DefinitionCreatedEventArgs : EventArgs
	{
		public string Name;
	}
	
	public delegate void ItemSelectedHandler (object o, ItemSelectedEventArgs args);
	public delegate void TestCreatedHandler (object o, TestCreatedEventArgs args);
	public delegate void DefinitionCreatedHandler (object o, DefinitionCreatedEventArgs args);
}
