// 
// sussen-editor - Main.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Text;
using Sussen.Oval;
using Sussen.Editor.Gui;

namespace Sussen.Editor
{
	public class Global
	{
		public static bool Debug;
	}

	public class MainApp
	{
		static void PrintUsage ()
		{
			StringBuilder usage = new StringBuilder ();
			usage.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			usage.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			usage.AppendLine ();
			usage.AppendFormat ("{0} [OPTIONS]\n\n", Defines.PACKAGE);
			usage.AppendLine ("Options:");
			usage.AppendLine ("  -d,  --debug\t\t\tDebug mode.");
			usage.AppendLine ("  -h,  --help\t\t\tPrint this usage message.");
			usage.AppendLine ("  -v,  --version\t\tPrint version and exit.");
						
			Console.WriteLine (usage.ToString());
		}

		static void PrintVersion ()
		{
			StringBuilder version = new StringBuilder ();
			version.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			version.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			version.AppendLine ("This is free software; see the source for copying conditions. There is NO");
			version.AppendLine ("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.");
		
			Console.WriteLine (version.ToString());
		}

		static void Main (string[] args) 
		{
			for (int i = 0; i < args.Length; i++) {

				string arg = args[i];

				switch (arg) {

				case "-d":
				case "--debug":
					Global.Debug = true;
					break;
				
				case "-h":
				case "--help":
					PrintUsage ();
					Environment.Exit (0);
					break;

				case "-v":
				case "--version":
					PrintVersion ();
					Environment.Exit (0);
					break;
				}
			}

			if (Global.Debug)
				Log.Initialize (LogLevel.Debug);
			else 
				Log.Initialize (LogLevel.Error);
			
			Log.Debug ("Starting {0} ({1})", Defines.PACKAGE, Defines.VERSION);
			
			Gtk.Application.Init ();
			
			GtkEditorWindow mainWindow = new GtkEditorWindow ();
			mainWindow.Run ();
			
			Log.Debug ("Exiting {0}", Defines.PACKAGE);
		}
	}
}
