// 
// sussen-editor - DefinitionView.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Xml;
using GLib;
using Gtk;
using Sussen.Oval;

namespace Sussen.Editor.Gui
{
	public class TestEditedEventArgs : EventArgs
	{
		public string ID;
	}

	public delegate void TestEditedHandler (object o, TestEditedEventArgs args);

	public class TestView : ScrolledWindow
	{
		Entry idEntry;
		Entry commentEntry;
		Entry typeEntry;

		TreeView objectTreeView;
		TreeView stateTreeView;
		
		ListStore objectStore;
		ListStore stateStore;
		
		ComboBox checkCombo;

		TestElement te;
		ObjectElement oe;
		StateElement se;
		
		bool firstTime;
		
		enum PopupType { Object, State };
	
		public event TestEditedHandler TestEdited;
	
		public TestView() : base () 
		{
			VBox rv = new VBox ();
					
			HBox h = new HBox ();
			h.BorderWidth = 6;
			h.Spacing = 6;
			rv.PackStart (h, false, false, 0);
			
			VBox v = new VBox ();
			v.Spacing = 6;
			h.PackStart (v, false, false, 0);
	
			Label l = new Label ("ID:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Comment:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Check:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);

			l = new Label ("Type:");
			l.Xalign = 0;
			v.PackStart (l, true, false, 0);
			
			v = new VBox ();
			v.Spacing = 6;
			h.PackStart (v, true, true, 0);

			idEntry = new Entry();
			idEntry.Changed += OnIdChanged;
			
			commentEntry = new Entry();
			commentEntry.Changed += OnCommentChanged;
			
			typeEntry = new Entry ();
			typeEntry.Sensitive = false;
			
			v.PackStart (idEntry, true, true, 0);
			v.PackStart (commentEntry, true, true, 0);
			
			checkCombo = ComboBox.NewText ();
			checkCombo.AppendText ("All");
			checkCombo.AppendText ("At least one");
			checkCombo.AppendText ("Only one");
			checkCombo.AppendText ("None exist");
			checkCombo.Active = 0;
			checkCombo.Changed += OnCheckChanged;
			checkCombo.Show ();			
			v.PackStart (checkCombo, true, true, 0);			
			
			v.PackStart (typeEntry, true, true, 0);
			
			VBox v2 = new VBox ();
			rv.PackEnd (v2, true, true, 0);
			
			HBox h2 = new HBox ();
			v2.PackStart (h2, false, false, 0);
			l = new Label ();
			l.UseMarkup = true;
			l.Markup = "<b>Object(s) to test</b>";
			h2.PackStart (l, false, false, 5);			

			h2 = new HBox ();
			v2.PackStart (h2, true, true, 5);
			
			ScrolledWindow sw = new ScrolledWindow ();
			sw.Show ();

			objectStore = new ListStore (typeof (string), typeof (string), typeof (string));			
			objectTreeView = new TreeView ();
			objectTreeView.Model = objectStore;
			objectTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			objectTreeView.AppendColumn ("Operation", new CellRendererText (), "text", 1);
			
			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnObjectEdit);			
			objectTreeView.AppendColumn ("Value", cell, "text", 2);
			
			objectTreeView.ButtonPressEvent += new ButtonPressEventHandler (OnObjectRightClick);
			
			sw.AddWithViewport (objectTreeView);
			h2.PackStart (sw, true, true, 5);
			
			h2 = new HBox ();
			v2.PackStart (h2, false, false, 0);
			l = new Label ();
			l.UseMarkup = true;
			l.Markup = "<b>State(s) to check for</b>";
			h2.PackStart (l, false, false, 5);

			h2 = new HBox ();
			v2.PackStart (h2, true, true, 5);
			
			sw = new ScrolledWindow ();
			sw.Show ();
			
			stateStore = new ListStore (typeof (string), typeof (string), typeof (string));
			stateTreeView = new TreeView ();
			stateTreeView.Model = stateStore;
			stateTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			stateTreeView.AppendColumn ("Operation", new CellRendererText (), "text", 1);
			
			cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnStateEdit);
			stateTreeView.AppendColumn ("Value", cell, "text", 2);
			stateTreeView.ButtonPressEvent += new ButtonPressEventHandler (OnStateRightClick);
			
			sw.AddWithViewport (stateTreeView);
			h2.PackStart (sw, true, true, 5);			
			
			this.AddWithViewport (rv);
			this.ShowAll ();
		}
		
		public void ShowTest (TestElement te, ObjectElement oe, StateElement se)
		{
			firstTime = true;
			
			idEntry.Text = te.ID;			
			commentEntry.Text = te.Comment;
			SetCheck (te.Check);
			typeEntry.Text = Vocabulary.GetTestName (te.Name);
			
			objectStore.Clear ();
			foreach (string k in oe.Entities.Keys) {
				EntityObject eo = oe.Entities [k];
				objectStore.AppendValues (eo.Name, Vocabulary.GetOperation (eo.Operation), eo.Value);
			}
			
			stateStore.Clear ();
			foreach (string k in se.Entities.Keys) {
				EntityObject eo = se.Entities [k];
				stateStore.AppendValues (eo.Name, Vocabulary.GetOperation (eo.Operation), eo.Value);
			}
			
			Test test = Interpreter.CreateTest (te.Name, "5.2");
			
			if (oe.Entities.Count == 0) {
				foreach (string s in test.ObjectNames)
					objectStore.AppendValues (s, "equals", "");
			}			
			
			foreach (string s in test.StateNames) {
				if (!se.Entities.ContainsKey (s))
					stateStore.AppendValues (s, "", "");
			}
			
			this.te = te;
			this.oe = oe;
			this.se = se;
			
			firstTime = false;
		}
			
		Checks GetCheck ()
		{
			TreeIter iter;
			checkCombo.GetActiveIter (out iter);

			string checkName = (string) checkCombo.Model.GetValue (iter, 0);
			Checks checkType = Vocabulary.GetCheck (checkName);
			
			return checkType;
		}
		
		void SetCheck (Checks testCheck) 
		{
			switch (testCheck) {
			case Checks.All:
				checkCombo.Active = 0;
				break;
			
			case Checks.AtLeastOne:
				checkCombo.Active = 1;
				break;
						
			case Checks.OnlyOne:
				checkCombo.Active = 2;
				break;

			case Checks.NoneExist:
				checkCombo.Active = 3;
				break;
				
			default:
				break;
			}
		}
		
		void CreateObjectPopup (Gtk.Menu typeMenu)
		{
			TreeModel model;
			TreeIter iter;

			if (objectTreeView.Selection.GetSelected (out model, out iter)) {
	
				string name = (string) objectStore.GetValue (iter, 0);
				string currentOp = (string) objectStore.GetValue (iter, 1);
				
				string testName = Vocabulary.GetTestType (typeEntry.Text);
				Test test = Interpreter.CreateTest (testName, "5.2");
			
				foreach (Operations o in test.GetObjectOperations (name)) {				
					string op = Vocabulary.GetOperation (o);
					if (op == currentOp)
						continue;
				
					MenuItem item = new MenuItem (op);
					item.Activated += new EventHandler (OnObjectOperationChange);
					item.Show ();
					typeMenu.Append (item);								
				}
			}		
		}

		void CreateStatePopup (Gtk.Menu typeMenu)
		{
			TreeModel model;
			TreeIter iter;

			if (stateTreeView.Selection.GetSelected (out model, out iter)) {
	
				string name = (string) stateStore.GetValue (iter, 0);
				string currentOp = (string) stateStore.GetValue (iter, 1);
				
				string testName = Vocabulary.GetTestType (typeEntry.Text);
				Test test = Interpreter.CreateTest (testName, "5.2");
			
				foreach (Operations o in test.GetStateOperations (name)) {				
					string op = Vocabulary.GetOperation (o);
					if (op == currentOp)
						continue;
				
					MenuItem item = new MenuItem (op);
					item.Activated += new EventHandler (OnStateOperationChange);
					item.Show ();
					typeMenu.Append (item);								
				}
			}		
		}
		
		void DoPopUp(PopupType popupType)
		{
			Menu popup = new Menu();

			MenuItem changeItem = new MenuItem ("Change operation");
			changeItem.Show ();
			popup.Append (changeItem);

			Gtk.Menu typeMenu = new Gtk.Menu ();

			if (popupType == PopupType.Object)
				CreateObjectPopup (typeMenu);
			else
				CreateStatePopup (typeMenu);

			changeItem.Submenu = typeMenu;				
			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}
		
		void OnObjectEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (firstTime)
				return;

			if (!objectStore.GetIterFromString (out iter, args.Path))
				return;

			string oldText = (string) objectStore.GetValue (iter, 2);
			if (oldText == args.NewText)
				return;
				
			objectStore.SetValue (iter, 2, args.NewText);

			string name = (string) objectStore.GetValue (iter, 0);
			if (oe.Entities.ContainsKey (name)) {
				EntityObject eo = oe.Entities[name];
				eo.Value = args.NewText;
			} else {
				// FIXME: need to get the correct default datatype
				EntityObject eo = new EntityObject (name, args.NewText, Operations.Equals, OvalDataTypes.StringType, Checks.All);
				oe.Entities.Add (name, eo);
			}
		}

		void DispatchTestEditedEvent ()
		{		
			if (TestEdited != null) {
				TestEditedEventArgs targs = new TestEditedEventArgs ();
				targs.ID = te.ID;
				
				TestEdited (this, targs);
			}
		}

		void OnIdChanged (object o, EventArgs args)
		{
			if (te != null && !firstTime) {
				te.ID = idEntry.Text;
				DispatchTestEditedEvent ();
			}
		}
		
		void OnCommentChanged (object o, EventArgs args)
		{
			if (te != null && !firstTime) {
				te.Comment = commentEntry.Text;
				DispatchTestEditedEvent ();
			}
		}

		void OnCheckChanged (object o, EventArgs args)
		{
			if (te != null && !firstTime) {
				te.Check = GetCheck ();
				DispatchTestEditedEvent ();
			}
		}
		
		void OnObjectOperationChange (object o, EventArgs args)
		{
			MenuItem mi = (MenuItem) o;
			Label l = (Label) mi.Child;

			TreeModel model;
			TreeIter iter;

			if (objectTreeView.Selection.GetSelected (out model, out iter)) {
	
				string name = (string) objectStore.GetValue (iter, 0);
				string currentOp = (string) objectStore.GetValue (iter, 1);
			
				Sussen.Oval.Log.Debug ("Changing {0} operation from {1} to {2}", name, currentOp, l.Text);				
				objectStore.SetValue (iter, 1, l.Text);

				EntityObject eo = oe.Entities[name];
				eo.Operation = Vocabulary.GetOperation (l.Text);
			
				DispatchTestEditedEvent ();						
			}
		}

		void OnStateOperationChange (object o, EventArgs args)
		{
			MenuItem mi = (MenuItem) o;
			Label l = (Label) mi.Child;

			TreeModel model;
			TreeIter iter;

			if (stateTreeView.Selection.GetSelected (out model, out iter)) {
	
				string name = (string) stateStore.GetValue (iter, 0);
				string currentOp = (string) stateStore.GetValue (iter, 1);
			
				Sussen.Oval.Log.Debug ("Changing {0} operation from {1} to {2}", name, currentOp, l.Text);				
				stateStore.SetValue (iter, 1, l.Text);				
				
				EntityObject eo = se.Entities[name];
				eo.Operation = Vocabulary.GetOperation (l.Text);
				
				DispatchTestEditedEvent ();				
			}
		}
		
		[ConnectBefore]
		void OnObjectRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp (PopupType.Object);
		}
		
		[ConnectBefore]
		void OnStateRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp (PopupType.State);
		}		
		
		void OnStateEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (firstTime)
				return;

			if (!stateStore.GetIterFromString (out iter, args.Path))
				return;

			string oldText = (string) stateStore.GetValue (iter, 2);
			if (oldText == args.NewText)
				return;
				
			stateStore.SetValue (iter, 2, args.NewText);

			string name = (string) stateStore.GetValue (iter, 0);
			if (se.Entities.ContainsKey (name)) {
				EntityObject eo = se.Entities[name];
				eo.Value = args.NewText;
			} else {
				// FIXME: need to get the correct default datatype
				EntityObject eo = new EntityObject (name, args.NewText, Operations.Equals, OvalDataTypes.StringType, Checks.All);
				se.Entities.Add (name, eo);
			}
		}		
	}
}
