// 
// HttpdTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Sussen.Oval;

namespace Sussen.Oval.Apache
{
	public class HttpdTest : Test
	{
		public HttpdTest () : base ()
		{
			Operations[] ops = { 
				Operations.Equals, 
				Operations.NotEqual, 
				Operations.GreaterThan,
				Operations.GreaterThanOrEqual,
				Operations.LessThan,
				Operations.LessThanOrEqual
			};

			Operations[] ops2 = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			
			this.stateData.Add ("path", ops2);
			this.stateData.Add ("binary_name", ops2);
			this.stateData.Add ("version", ops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			items.Add (CollectData ());
			
			return items.ToArray ();
		}

		CollectedItem CollectData ()
		{
			CollectedItem item = new CollectedItem ("httpd_item", 0, Vocabulary.APACHE_NAMESPACE, Flags.NotCollected);
			
			// FIXME: Need to be able to search paths for all potential apache binaries; don't assume just one
			string[] commonPaths = { "/usr/sbin/httpd", "/usr/sbin/apache2", "c:\apache group\apache2\bin\apache.exe" };
			string output = CmdReader.Read (commonPaths, "-v");

			if (output == null) {
				item.Flag = Flags.Error;
				item.Message = "Unable to determine apache version";
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("version", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				return item;
			}

			char[] newlineSplit = {'\n'};
			string[] tmp = output.Split (newlineSplit);

			char[] colonSplit = {':'};
			string[] tmp2 = tmp[0].Split (colonSplit);

			char[] slashSplit = {'/'};
			string[] tmp3 = tmp2[1].Split (slashSplit);

			item.Flag = Flags.Complete;
			item.AddItem (new EntityObject ("path", "FIXME", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("binary_name", "FIXME", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("version", tmp3[1], OvalDataTypes.StringType, StatusFlags.Exists, false));
			
			return item;
		}
	}
}
