// 
// FamilyTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Xml;

namespace Sussen.Oval.Independent
{
	public class FamilyTest : Test
	{
		public FamilyTest () : base ()
		{
			Operations[] ops = { 
				Operations.Equals, 
				Operations.NotEqual, 
				Operations.PatternMatch
			};
		
			this.stateData.Add ("family", ops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
						
			items.Add (CollectData ());
			
			return items.ToArray ();
		}

		CollectedItem CollectData ()
		{
			CollectedItem item = new CollectedItem ("family_item", 0, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);
			
			string versionString = Environment.OSVersion.ToString();
			versionString = versionString.ToLower ();
			string famName = null;

			if (versionString.StartsWith ("unix"))
				famName = "unix";
			else if (versionString.StartsWith ("microsoft"))
				famName = "windows";				
			else
				famName = "unknown";

			if (File.Exists ("/etc/lsb-release"))
				famName = "linux";
		
			item.AddItem (new EntityObject ("family", famName, OvalDataTypes.StringType, StatusFlags.Exists, false));
			return item;
		}
	}
}
