// 
// VariableTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval.Independent
{
	public class VariableTest : Test
	{
		public VariableTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("var_ref", ops);
			
			// FIXME: value can be any type (bool, string, etc.)
			this.stateData.Add ("var_ref", ops);
			this.stateData.Add ("value", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("var_ref"))
				throw new TestException (testElement.ID, "Missing name object from variable test");

			EntityObject varName = objectElement.Entities["var_ref"];

			if (varName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for name object in variable test");

			switch (varName.Operation) {
			
			case Operations.Equals:
				items.Add (CollectData (varName.Value));
				break;
				
			case Operations.NotEqual:
			case Operations.PatternMatch:
				string[] foundVars = FindVariables (varName);
				if (foundVars.Length == 0)
					items.Add (CreateNotFound (varName.Value));
					
				foreach (string v in foundVars) {
					items.Add (CollectData (v));
					itemCount++;
				}
				break;
			
			default:
				throw new TestException (testElement.ID, 
					"Invalid operation for variable test: " + Vocabulary.GetOperation (varName.Operation));
			}
			
			return items.ToArray ();
		}

		string[] FindVariables (EntityObject varName)
		{
			List<string> items = new List<string> ();
		
			foreach (KeyValuePair<string, Variable> kvp in Global.Variables) {
				Variable v = kvp.Value;
				if (CompareItems (varName.Value, v.ID, varName.Operation))
					items.Add (v.ID);
			}
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string varName)
		{
			CollectedItem item = new CollectedItem ("variable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("var_ref", varName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("value", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string varName)
		{
			CollectedItem item = new CollectedItem ("variable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);
			
			Variable v = Global.Variables [varName];
			if (v == null)
				return CreateNotFound (varName);

			item.AddItem (new EntityObject ("var_ref", varName, OvalDataTypes.StringType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("value", v.Value, v.DataType, StatusFlags.Exists, false));			
			
			return item;
		}
	}
}
