// 
// SccsTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class SccsTest : Test
	{
		public SccsTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("path", ops);
			this.objectData.Add ("filename", ops);
			
			this.stateData.Add ("path", ops);
			this.stateData.Add ("filename", ops);
			this.stateData.Add ("module_name", ops);
			this.stateData.Add ("module_type", ops);
			this.stateData.Add ("release", ops);
			this.stateData.Add ("level", ops);
			this.stateData.Add ("branch", ops);
			this.stateData.Add ("sequence", ops);
			this.stateData.Add ("what_string", ops);		
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("path"))
				throw new TestException (testElement.ID, "Invalid sccs test; missing path");
				
			if (!objectElement.Entities.ContainsKey ("filename"))
				throw new TestException (testElement.ID, "Invalid sccs test; missing filename");
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for path in file test");

			if (file.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for filename in file test");

			CheckEntityOperation (path);
			CheckEntityOperation (file);

			// FIXME: Don't know what to collect for this test		
			items.Add (CollectData (path.Value, file.Value));
			
			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in sccs test");
			}		
		}

		CollectedItem CollectData (string filePath, string fileName)
		{
			CollectedItem item = new CollectedItem ("sccs_item", 0, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected);
			item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.NotCollected, true));
			item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.NotCollected, true));		
			item.AddItem (new EntityObject ("module_name", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("module_type", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("release", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("level", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("branch", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("sequence", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("what_string", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			
			return item;
		}		
	}
}
