// 
// SystemInfoTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class UnixSystemInfo : SystemInfo
	{
		string osName;
		string osVersion;
		string osArch;
		string primaryHostName;
		ArrayList interfaces;
	
		public UnixSystemInfo ()
		{
			osName = "";
			osVersion = "";
			osArch = "";
			primaryHostName = "";
			
			interfaces = new ArrayList ();			
			UnameTest ut = new UnameTest ();
			
			CollectedItem[] ud = ut.Run (null, null);
			CollectedItem item = ud[0];
			
			foreach (EntityObject ie in item.Items) {
				if (ie.Name == "os_name")
					osName = ie.Value;
				else if (ie.Name == "os_version")
					osVersion = ie.Value;
				else if (ie.Name == "machine_class")
					osArch = ie.Value;
			}
			
			GetNetworkInterfaces ();			
		}		
		
		public override string OsName
		{
			get { return osName; }
		}
		
		public override string OsVersion
		{
			get { return osVersion; }
		}
		
		public override string OsArchitecture
		{
			get { return osArch; }
		}
		
		public override string PrimaryHostName
		{
			get { return primaryHostName; }
		}
		
		public override SystemInterfaceData[] Interfaces
		{
			get { return (SystemInterfaceData[]) interfaces.ToArray (typeof (SystemInterfaceData)); }
		}
		
		[DllImport("libsussensupport")]
		internal extern static int get_network_interface_names (out string csvList);

		[DllImport("libsussensupport")]
		internal extern static int get_network_interface_info (string ifname,
			out string hw_addr, out string inet_addr, out string bc_addr,
			out string netmask, out short flags);

		SystemInterfaceData GetInterfaceInfo (string interfaceName)
		{
			string hw_addr, inet_addr, bc_addr, netmask;
			short flags;

			SystemInterfaceData id = new SystemInterfaceData ();
			id.Name = interfaceName;

			int rc = get_network_interface_info (interfaceName,
				out hw_addr, out inet_addr, out bc_addr, out netmask, out flags);

			if (rc == 0) {
				id.MacAddress = hw_addr;
				id.IpAddress = inet_addr;
			} else {
				id.MacAddress = "Unknown";
				id.IpAddress = "Unknown";
			}
		
			return id;
		}

		void GetNetworkInterfaces ()
		{
			try {
			
				IPHostEntry hostInfo = Dns.GetHostEntry(Dns.GetHostName());				
				primaryHostName = hostInfo.HostName;

			} catch (Exception e) {
				Log.Debug (e);
				primaryHostName = "Unknown";
			}


			string lst;
			int rc = get_network_interface_names (out lst);

			if (rc == 0) {
				char[] splitter  = {','};
				string[] ifn;
				ifn = lst.Split (splitter);

				foreach (string n in ifn) {
					SystemInterfaceData id = GetInterfaceInfo (n);
					interfaces.Add (id);
				}
			}
		}
	}
}
