// 
// UnameTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class UnameTest : Test
	{
		public UnameTest ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
				
			this.stateData.Add ("machine_class", ops);
			this.stateData.Add ("node_name", ops);
			this.stateData.Add ("os_name", ops);
			this.stateData.Add ("os_release", ops);
			this.stateData.Add ("os_version", ops);
			this.stateData.Add ("processor_type", ops);		
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			items.Add (CollectData ());
			
			return items.ToArray ();
		}

		CollectedItem CollectData ()
		{
			CollectedItem item = new CollectedItem ("uname_item", 0, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected);

			string [] paths = new string[] { "/bin/uname", "/usr/bin/uname" };				
			string nameCheck = CmdReader.Read (paths, "-m");
				
			if (nameCheck != null) {
		
				item.Flag = Flags.Complete;
				item.AddItem (new EntityObject ("machine_class", CmdReader.Read (paths, "-m"), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("node_name", CmdReader.Read (paths, "-n"), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("os_name", CmdReader.Read (paths, "-s"), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("os_release", CmdReader.Read (paths, "-r"), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("os_version", CmdReader.Read (paths, "-v"), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("processor_type", CmdReader.Read (paths, "-p"), OvalDataTypes.StringType, StatusFlags.Exists, false));
									
			} else {

				item.Flag = Flags.Error;
				item.Message = "Unable to get uname information";
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("machine_class", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("node_name", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("os_name", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("os_release", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("os_version", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("processor_type", "", OvalDataTypes.StringType, StatusFlags.Error, false));

			}			
			
			return item;
		}
	}
}
