// 
// DataAnalyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Xml;

namespace Sussen.Oval
{
	public class DataAnalyzer : Analyzer
	{
		OvalDocuments docs;
		
		public DataAnalyzer (OvalDocuments ovalDocuments)
		{
			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");		
			
			docs = ovalDocuments;
			docs.Results.CopyDefinitions (docs.Definitions);
			docs.Results.CopySystemCharacteristics (docs.SystemCharacteristics);
		}
		
		public void Run ()
		{
			XmlNodeList ovalDefinitions = docs.Definitions.GetElementsByName ("definition"); 			
			if (!(ovalDefinitions.Count > 0))
				throw new AnalyzerException ("No definitions found in document");
				
			foreach (XmlElement def in ovalDefinitions) {
				string id = def.GetAttribute ("id");
				TestResult res = TestResult.Unknown;
				
				Log.Debug ("Starting analysis on definition [{0}]", id);
				
				try {
				
					DefinitionAnalyzer da = new DefinitionAnalyzer (def, docs);		
					res = da.Run ();
					
				} catch (Exception e) {
					
					Log.Warn ("Error analyzing definition [{0}]. {1}", id, e.Message);
					Log.Debug (e);
				
				}
				
				Log.Debug ("Finished analysis on definition [{0}] result [{1}]", id, res);
				Log.Debug ("");
			}
		}
	}
}
