// 
// EvrStringType.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Text.RegularExpressions;

namespace Sussen.Oval 
{
	public class EvrStringType
	{
		int epoch;
		VersionType version;
		string release;

		const string epochPattern = @"[0-9]:[0-9\.]+-[A-Za-z0-9_\-\.]*";

		public EvrStringType (string evrString)
		{
			char[] versionReleaseSep = {'-'};
			char[] epochVersionSep = {':'};

			Match m = Regex.Match (evrString, epochPattern);
			if (m.Success) {

				string[] eparts = evrString.Split (epochVersionSep);
				string tmpVersion = eparts[1];
				string[] vparts = tmpVersion.Split (versionReleaseSep);

				this.epoch = int.Parse (eparts[0]);
				this.version = new VersionType (vparts[0]);

				if (vparts.Length == 2)
					this.release = vparts[1];
				else
	 				this.release = "";
			} else {
				string[] vparts = evrString.Split (versionReleaseSep);			
				this.version = new VersionType (vparts[0]);

				if (vparts.Length == 2)
					this.release = vparts[1];
				else
	 				this.release = "";
			}
		}

		public EvrStringType (string epoch, string version, string release)
		{
			this.epoch = int.Parse (epoch);
			this.version = new VersionType (version);
			this.release = release;
		}


		public override bool Equals (Object o)
		{
			if (o is EvrStringType) {
				EvrStringType e2 = (EvrStringType) o;

				if (this == e2)
					return true;
				else 
					return false;
			}

			return false;
		}

		public override int GetHashCode ()
		{
			return 0;
		}

		public static bool operator == (EvrStringType e1, EvrStringType e2)
		{
			if (e1.Epoch != e2.Epoch)
				return false;

			if (e1.Version != e2.Version)
				return false;

			if (e1.Release.Length != e2.Release.Length)
				return false;

			char[] e1Chars = e1.Release.ToCharArray ();
			char[] e2Chars = e2.Release.ToCharArray ();

			bool allEqual = true;
			for (int i = 0; i < e1.Release.Length; i++) {
				if (Char.IsDigit (e1Chars[i]) && Char.IsDigit (e2Chars[i])) {
					int a = int.Parse (e1Chars[i].ToString());
					int b = int.Parse (e2Chars[i].ToString());

					if (a != b)
						allEqual = false;
				} else {
					if (e1Chars[i] != e2Chars[i])
						allEqual = false;
				}
			} 

			return allEqual;
		}

		public static bool operator != (EvrStringType e1, EvrStringType e2)
		{
			if (e1 == e2)
				return false;

			return true;
		}

		public static bool operator > (EvrStringType e1, EvrStringType e2)
		{
			bool greaterThan = false;

			if (e1.Epoch > e2.Epoch)
				return true;

			if (e1.Version > e2.Version)
				return true;

			char[] e1Chars = e1.Release.ToCharArray ();
			char[] e2Chars = e2.Release.ToCharArray ();

			if (e1Chars.Length > e2Chars.Length)
				return true;
			else if (e2Chars.Length > e1Chars.Length)
				return false;

			for (int i = 0; i < e1.Release.Length; i++) {
				if (Char.IsDigit (e1Chars[i]) && Char.IsDigit (e2Chars[i])) {
					int a = int.Parse (e1Chars[i].ToString());
					int b = int.Parse (e2Chars[i].ToString());

					if (a > b)
						greaterThan = true;
				}
			} 

			return greaterThan;		
		}

		public static bool operator >= (EvrStringType e1, EvrStringType e2)
		{
			bool greaterThanEqual = false;

			if (e1.Epoch > e2.Epoch)
				return true;

			if (e1.Version > e2.Version)
				return true;

			char[] e1Chars = e1.Release.ToCharArray ();
			char[] e2Chars = e2.Release.ToCharArray ();

			if (e1Chars.Length > e2Chars.Length)
				return true;
			else if (e2Chars.Length > e1Chars.Length)
				return false;

			for (int i = 0; i < e1.Release.Length; i++) {
				if (Char.IsDigit (e1Chars[i]) && Char.IsDigit (e2Chars[i])) {
					int a = int.Parse (e1Chars[i].ToString());
					int b = int.Parse (e2Chars[i].ToString());

					if (a >= b)
						greaterThanEqual = true;
				}
			}

			return greaterThanEqual;		
		}

		public static bool operator < (EvrStringType e1, EvrStringType e2)
		{
			bool lessThan = false;

			if (e1.Epoch < e2.Epoch)
				return true;

			if (e1.Version < e2.Version)
				return true;

			char[] e1Chars = e1.Release.ToCharArray ();
			char[] e2Chars = e2.Release.ToCharArray ();

			if (e1Chars.Length > e2Chars.Length)
				return false;
			else if (e2Chars.Length > e1Chars.Length)
				return false;

			for (int i = 0; i < e1Chars.Length; i++) {
				if (Char.IsDigit (e1Chars[i]) && Char.IsDigit (e2Chars[i])) {
					int a = int.Parse (e1Chars[i].ToString());
					int b = int.Parse (e2Chars[i].ToString());

					if (a < b)
						lessThan = true;
				}
			} 

			return lessThan;		
		}

		public static bool operator <= (EvrStringType e1, EvrStringType e2)
		{
			bool lessThanEqual = false;

			if (e1.Epoch < e2.Epoch)
				return true;

			if (e1.Version < e2.Version)
				return true;

			char[] e1Chars = e1.Release.ToCharArray ();
			char[] e2Chars = e2.Release.ToCharArray ();

			if (e1Chars.Length > e2Chars.Length)
				return false;
			else if (e2Chars.Length > e1Chars.Length)
				return false;

			for (int i = 0; i < e1.Release.Length; i++) {
				if (Char.IsDigit (e1Chars[i]) && Char.IsDigit (e2Chars[i])) {
					int a = int.Parse (e1Chars[i].ToString());
					int b = int.Parse (e2Chars[i].ToString());

					if (a <= b)
						lessThanEqual = true;
				}
			} 

			return lessThanEqual;
		}

		public int Epoch
		{
			get { return epoch; }
		}

		public VersionType Version
		{
			get { return version; }
		}

		public string Release
		{
			get { return release; }
		}
	}
}