// 
// VersionType.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;

namespace Sussen.Oval 
{
	public class VersionType
	{
		int[] digits;

		public VersionType (string versionString)
		{	
			string[] tmp = versionString.Split(new char[] { ' ' });
			string vs = tmp[0];
            
			char[] seperators = new char[] {'.','-'};
			string[] res = vs.Split (seperators);

			digits = new int[res.Length];

			int c = res.Length - 1;
			foreach (string s in res)
			{
				try {
					digits[c] = int.Parse (s);
				} catch {}
				
				c--;
			}
		}

		public void AddPadding(int size)
        {
            List<int> newDigits = new List<int>();

            for (int i = 0; i < size; i++)
                newDigits.Add(0);

            foreach (int i in digits) 
                newDigits.Add(i);

            digits = newDigits.ToArray();
        }

		public override bool Equals (Object o)
		{
			if (o is VersionType) {
				VersionType v2 = (VersionType) o;

				if (this == v2)
					return true;
				else 
					return false;
			}

			return false;
		}

		public override int GetHashCode ()
		{
			int retVal = 0;
			for (int i = 0; i < this.digits.Length; i++)
				retVal += this.digits[i];

			return retVal;
		}

		public static bool operator == (VersionType v1, VersionType v2)
		{
			if (v1.Digits.Length != v2.Digits.Length)
				v2.AddPadding (v1.Digits.Length - v2.Digits.Length); 

			bool allEqual = true;

			for (int i = (v1.Digits.Length - 1); i >= 0; i--) {
				if (v1.Digits[i] != v2.Digits[i]) {
					allEqual = false;
					break;
				}
			}

			return allEqual;
		}

		public static bool operator != (VersionType v1, VersionType v2)
		{
			if (v1.Digits.Length != v2.Digits.Length)
				return true;

			if (v1 == v2)
				return false;

			return true;
		}

		public static bool operator > (VersionType v1, VersionType v2)
		{
			bool greaterThan = false;
            
			if (v1.Digits.Length != v2.Digits.Length)
				v2.AddPadding (v1.Digits.Length - v2.Digits.Length);

			for (int i = (v1.Digits.Length - 1); i >= 0; i--) {
				if (v1.Digits[i] == v2.Digits[i])
					continue;

				if (v1.Digits[i] > v2.Digits[i]) {
					greaterThan = true;
					break;
				} else {
					break;
				}
			}

			return greaterThan;		
		}

		public static bool operator >= (VersionType v1, VersionType v2)
		{
			bool greaterThanEqual = true;

			if (v1.Digits.Length != v2.Digits.Length)
				v2.AddPadding (v1.Digits.Length - v2.Digits.Length);

			for (int i = (v1.Digits.Length - 1); i >= 0; i--) {
				if (v1.Digits[i] >= v2.Digits[i])
					continue;
				else
				{
					greaterThanEqual = false;
					break;
				}
			}

			return greaterThanEqual;		
		}

		public static bool operator < (VersionType v1, VersionType v2)
		{
			bool lessThan = false;

			if (v1.Digits.Length != v2.Digits.Length)
				v2.AddPadding (v1.Digits.Length - v2.Digits.Length);

			for (int i = (v1.Digits.Length - 1); i >= 0; i--) {
				if (v1.Digits[i] == v2.Digits[i])
					continue;

				if (v1.Digits[i] < v2.Digits[i]) {
					lessThan = true;
					break;
				} else { 
					break;
				}
			}

			return lessThan;
		}

		public static bool operator <= (VersionType v1, VersionType v2)
		{
			bool lessThanEqual = true;

			if (v1.Digits.Length != v2.Digits.Length)
				v2.AddPadding (v1.Digits.Length - v2.Digits.Length);

			for (int i = (v1.Digits.Length - 1); i >= 0; i--) {
				if (v1.Digits[i] <= v2.Digits[i])
				        continue;
				else
				{
				        lessThanEqual = false;
				        break;
				}
			}

			return lessThanEqual;
		}

		public int[] Digits
		{
			get { return digits; }
		}
	}
}
