/*
 * Code adapted from net-tools 1.60 by Phil Blundell and Bernd Eckenfels
 */

#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <stdio.h>
#include <string.h>

char *ntoa(unsigned long ip) {
        static char buff[18];
        char *p;
        p = (char *) &ip;
        sprintf(buff, "%d.%d.%d.%d",
                (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));
        return(buff);
}

int get_network_interface_info (char *ifname,
	char **phw_addr,
	char **pinet_addr,
	char **pbc_addr,
	char **pnetmask,
	short *pflags) {

	struct ifreq ifr;
	int sfd;

	char hw_addr[18];
	char inet_addr[18];
	char bc_addr[18];
	char netmask[18];
	short flags;
	
	if (ifname == NULL || strlen(ifname) <= 1) {
		*phw_addr = strdup (hw_addr);
		*pinet_addr = strdup (inet_addr);
		*pbc_addr = strdup (bc_addr);
		*pnetmask = strdup (netmask);
		*pflags = flags;

		return (-1);
	}

	sfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (sfd < 0) {
		*phw_addr = strdup (hw_addr);
		*pinet_addr = strdup (inet_addr);
		*pbc_addr = strdup (bc_addr);
		*pnetmask = strdup (netmask);
		*pflags = flags;

		return (-1);
	}

	strncpy (ifr.ifr_name, ifname, IFNAMSIZ);
	if (ioctl(sfd, SIOCGIFFLAGS, &ifr) < 0) {
		*phw_addr = strdup (hw_addr);
		*pinet_addr = strdup (inet_addr);
		*pbc_addr = strdup (bc_addr);
		*pnetmask = strdup (netmask);
		*pflags = flags;

		return (-1);
	} else {
		flags = ifr.ifr_flags;
	}

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	if (ioctl(sfd, SIOCGIFHWADDR, &ifr) < 0) {
		*phw_addr = strdup (hw_addr);
		*pinet_addr = strdup (inet_addr);
		*pbc_addr = strdup (bc_addr);
		*pnetmask = strdup (netmask);
		*pflags = flags;

		return (-1);
	} else {
	   snprintf (hw_addr, 18, "%02x:%02x:%02x:%02x:%02x:%02x", 
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[0],
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[1],
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[2],
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[3],
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[4],
		(int) ((unsigned char *) ifr.ifr_hwaddr.sa_data)[5]);
	}

	strcpy(ifr.ifr_name, ifname);
	ifr.ifr_addr.sa_family = AF_INET;
	if (ioctl(sfd, SIOCGIFADDR, &ifr) == 0) {
		sprintf (inet_addr, "%s", 
			ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr));

		strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
		if (ioctl(sfd, SIOCGIFBRDADDR, &ifr) < 0) {
			*phw_addr = strdup (hw_addr);
			*pinet_addr = strdup (inet_addr);
			*pbc_addr = strdup (bc_addr);
			*pnetmask = strdup (netmask);
			*pflags = flags;

			return (-1);
		} else {
			sprintf (bc_addr, "%s", 
				ntoa(((struct sockaddr_in *)&ifr.ifr_broadaddr)->sin_addr.s_addr));
		}

		strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
		if (ioctl(sfd, SIOCGIFNETMASK, &ifr) < 0) {
			*phw_addr = strdup (hw_addr);
			*pinet_addr = strdup (inet_addr);
			*pbc_addr = strdup (bc_addr);
			*pnetmask = strdup (netmask);
			*pflags = flags;

			return (-1);
		} else {
			sprintf (netmask, "%s", 
				ntoa(((struct sockaddr_in *)&ifr.ifr_netmask)->sin_addr.s_addr));
		}

	} else {
		*phw_addr = strdup (hw_addr);
		*pinet_addr = strdup (inet_addr);
		*pbc_addr = strdup (bc_addr);
		*pnetmask = strdup (netmask);
		*pflags = flags;

		return (-1);
	}

	*phw_addr = strdup (hw_addr);
	*pinet_addr = strdup (inet_addr);
	*pbc_addr = strdup (bc_addr);
	*pnetmask = strdup (netmask);
	*pflags = flags;

	return 0;	
}

static char *get_name(char *name, char *p)
{
    while (isspace(*p))
        p++;
    while (*p) {
        if (isspace(*p))
            break;
        if (*p == ':') {        /* could be an alias */
            char *dot = p, *dotname = name;
            *name++ = *p++;
            while (isdigit(*p))
                *name++ = *p++;
            if (*p != ':') {    /* it wasn't, backup */
                p = dot;
                name = dotname;
            }
            if (*p == '\0')
                return NULL;
            p++;
            break;
        }
        *name++ = *p++;
    }
    *name++ = '\0';
    return p;
}

int get_network_interface_names (char **found_interfaces)
{
	FILE *f;
	char buf[512];
        char interfaces [1024];

	f = fopen ("/proc/net/dev", "r");
	if (!f) {
		*found_interfaces = strdup (interfaces);
		return (-1);
	}
	
	fgets (buf, sizeof (buf), f); /* eat line */
	fgets (buf, sizeof (buf), f);

	int i = 0;
	while (fgets(buf, sizeof buf, f)) {
		char name[IFNAMSIZ];
		get_name (name, buf);
		int len = strlen (interfaces) + strlen (name) + 2;

		if (i == 0)
			snprintf (interfaces, IFNAMSIZ, "%s", name);
		else
			snprintf (interfaces, len, "%s,%s", strdup(interfaces), name);

		i++;
	}

	fclose (f);
	
	*found_interfaces = strdup (interfaces);

	return 0;
}

