//
// AnalyzeByCheckTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Collections.Generic;
using NUnit.Framework;
using Sussen.Oval;

namespace SussenTests.Oval
{
	[TestFixture]
	public class AnalyzeByCheckTest
	{
		[SetUp]
		public void SetUp()
		{
		}

		[TearDown]
		public void TearDown ()
		{
		}

		public List<TestResult> CreateResults (int trueCount, int falseCount, int errorCount, int unknownCount, int notEvaluatedCount, int notApplicableCount)
		{
			List<TestResult> results = new List<TestResult> ();

			for (int i = 0; i < trueCount; i++)
				results.Add (TestResult.True);

			for (int i = 0; i < falseCount; i++)
				results.Add (TestResult.False);

			for (int i = 0; i < errorCount; i++)
				results.Add (TestResult.Error);

			for (int i = 0; i < unknownCount; i++)
				results.Add (TestResult.Unknown);

			for (int i = 0; i < notEvaluatedCount; i++)
				results.Add (TestResult.NotEvaluated);

			for (int i = 0; i < notApplicableCount; i++)
				results.Add (TestResult.NotApplicable);

			return results;
		}

		[Test]
		public void TestAnalyzerByCheckStateMachine ()
		{
			// all states
			List<TestResult> allTrueResults = CreateResults (5, 0, 0, 0, 0, 0);
			TestResult res = DataAnalyzer.AnalyzeResultsByCheck (allTrueResults, "all");
			Assert.AreEqual (res, TestResult.True, "All true results failed");

			List<TestResult> allFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allFalseResults, "all");
			Assert.AreEqual (res,  TestResult.False, "All false results failed");

			List<TestResult> allErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allErrorResults, "all");
			Assert.AreEqual (res,  TestResult.Error, "All error results failed");

			List<TestResult> allUnknownResults = CreateResults (1, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allUnknownResults, "all");
			Assert.AreEqual (res,  TestResult.Unknown, "All unknown results failed");

			List<TestResult> allNEResults = CreateResults (1, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allNEResults, "all");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "All not evaluated results failed");

			List<TestResult> allNAResults = CreateResults (1, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (allNAResults, "all");
			Assert.AreEqual (res,  TestResult.NotApplicable, "All not applicable results failed");

			// at least one
			List<TestResult> aloTrueResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloTrueResults, "at least one");
			Assert.AreEqual (res,  TestResult.True, "At least one true result failed");

			List<TestResult> aloFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloFalseResults, "at least one");
			Assert.AreEqual (res,  TestResult.False, "At least one false result failed");

			List<TestResult> aloErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloErrorResults, "at least one");
			Assert.AreEqual (res,  TestResult.Error, "At least one error result failed");

			List<TestResult> aloUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloUnknownResults, "at least one");
			Assert.AreEqual (res,  TestResult.Unknown, "At least one unknown result failed");

			List<TestResult> aloNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloNEResults, "at least one");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "At least one not evaluated result failed");

			List<TestResult> aloNAResults = CreateResults (0, 0, 0, 0, 0, 2);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloNAResults, "at least one");
			Assert.AreEqual (res,  TestResult.NotApplicable, "At least one not applicable result failed");

			// only one
			List<TestResult> ooTrueResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooTrueResults, "only one");
			Assert.AreEqual (res,  TestResult.True, "Only one true result failed");

			List<TestResult> ooFalseResults = CreateResults (3, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooFalseResults, "only one");
			Assert.AreEqual (res,  TestResult.False, "Only one false result (none) failed");

			List<TestResult> ooFalseResults2 = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooFalseResults2, "only one");
			Assert.AreEqual (res,  TestResult.False, "Only one false result failed");

			List<TestResult> ooErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooErrorResults, "only one");
			Assert.AreEqual (res,  TestResult.Error, "Only one error result failed");

			List<TestResult> ooUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooUnknownResults, "only one");
			Assert.AreEqual (res,  TestResult.Unknown, "Only one unknown result failed");

			List<TestResult> ooNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooNEResults, "only one");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "Only one not evaluated result failed");

			List<TestResult> ooNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooNAResults, "only one");
			Assert.AreEqual (res,  TestResult.NotApplicable, "Only one not applicable result failed");

			// none
			List<TestResult> noneTrueResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneTrueResults, "none");
			Assert.AreEqual (res,  TestResult.True, "None true result failed");

			List<TestResult> noneFalseResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneFalseResults, "none");
			Assert.AreEqual (res,  TestResult.False, "None false result failed");

			List<TestResult> noneErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneErrorResults, "none");
			Assert.AreEqual (res,  TestResult.Error, "None error result failed");

			List<TestResult> noneUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneUnknownResults, "none");
			Assert.AreEqual (res,  TestResult.Unknown, "None unknown result failed");

			List<TestResult> noneNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneNEResults, "none");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "None not evaluated result failed");

			List<TestResult> noneNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneNAResults, "none");
			Assert.AreEqual (res,  TestResult.NotApplicable, "None not applicable result failed");
		}
	}
}
