//
// AnalyzeByOperatorTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Collections.Generic;
using NUnit.Framework;
using Sussen.Oval;

namespace SussenTests.Oval
{
	[TestFixture]
	public class AnalyzeByOperatorTest
	{
		[SetUp]
		public void SetUp()
		{
		}

		[TearDown]
		public void TearDown ()
		{
		}

		public List<TestResult> CreateResults (int trueCount, int falseCount, int errorCount, int unknownCount, int notEvaluatedCount, int notApplicableCount)
		{
			List<TestResult> results = new List<TestResult> ();

			for (int i = 0; i < trueCount; i++)
				results.Add (TestResult.True);

			for (int i = 0; i < falseCount; i++)
				results.Add (TestResult.False);

			for (int i = 0; i < errorCount; i++)
				results.Add (TestResult.Error);

			for (int i = 0; i < unknownCount; i++)
				results.Add (TestResult.Unknown);

			for (int i = 0; i < notEvaluatedCount; i++)
				results.Add (TestResult.NotEvaluated);

			for (int i = 0; i < notApplicableCount; i++)
				results.Add (TestResult.NotApplicable);

			return results;
		}

		[Test]
		public void TestAnalyzerByOperatorStateMachine ()
		{
			// AND states
			List<TestResult> andTrueResults = CreateResults (2, 0, 0, 0, 0, 0);
			TestResult res = DataAnalyzer.AnalyzeResults (andTrueResults, "AND");
			Assert.AreEqual (res,  TestResult.True, "AND true results failed");

			List<TestResult> andFalseResults = CreateResults (2, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andFalseResults, "AND");
			Assert.AreEqual (res,  TestResult.False, "AND false results failed");

			List<TestResult> andErrorResults = CreateResults (3, 0, 2, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andErrorResults, "AND");
			Assert.AreEqual (res,  TestResult.Error, "AND error results failed");

			List<TestResult> andUnknownResults = CreateResults (1, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andUnknownResults, "AND");
			Assert.AreEqual (res,  TestResult.Unknown, "AND unknown results failed");

			List<TestResult> andNEResults = CreateResults (1, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (andNEResults, "AND");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "AND not evaluated results failed");

			List<TestResult> andNAResults = CreateResults (1, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (andNAResults, "AND");
			Assert.AreEqual (res,  TestResult.NotApplicable, "AND not applicable results failed");

			// OR states
			List<TestResult> orTrueResults = CreateResults (2, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orTrueResults, "OR");
			Assert.AreEqual (res,  TestResult.True, "OR true results failed");

			List<TestResult> orFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orFalseResults, "OR");
			Assert.AreEqual (res,  TestResult.False, "OR false results failed");

			List<TestResult> orErrorResults = CreateResults (0, 0, 2, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orErrorResults, "OR");
			Assert.AreEqual (res,  TestResult.Error, "OR error results failed");

			List<TestResult> orUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orUnknownResults, "OR");
			Assert.AreEqual (res,  TestResult.Unknown, "OR unknown results failed");

			List<TestResult> orNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (orNEResults, "OR");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "OR not evaluated results failed");

			List<TestResult> orNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (orNAResults, "OR");
			Assert.AreEqual (res,  TestResult.NotApplicable, "OR not applicable results failed");

			// XOR states
			List<TestResult> xorTrueResults = CreateResults (3, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorTrueResults, "XOR");
			Assert.AreEqual (res,  TestResult.True, "XOR true results failed");

			List<TestResult> xorFalseResults = CreateResults (2, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorFalseResults, "XOR");
			Assert.AreEqual (res,  TestResult.False, "XOR false results failed");

			List<TestResult> xorErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorErrorResults, "XOR");
			Assert.AreEqual (res,  TestResult.Error, "XOR error results failed");

			List<TestResult> xorUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorUnknownResults, "XOR");
			Assert.AreEqual (res,  TestResult.Unknown, "XOR unknown results failed");

			List<TestResult> xorNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (xorNEResults, "XOR");
			Assert.AreEqual (res,  TestResult.NotEvaluated, "XOR not evaluated results failed");

			List<TestResult> xorNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (xorNAResults, "XOR");
			Assert.AreEqual (res,  TestResult.NotApplicable, "XOR not applicable results failed");
		}
	}
}
