//
// EvrString.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using NUnit.Framework;
using Sussen.Oval;

namespace SussenTests.Oval
{
	[TestFixture]
	public class EvrStringTests
	{
		const string e1 = "1:1.0.0-0ubuntu4";
		const string e2 = "1:1.0.0-0ubuntu3";
		const string e3 = "1:1.0.0-0ubuntu4";
		const string e4 = "1.5.0-2dog5";
		const string e5 = "1:1.0.0-0ubuntu40";
	
		[Test]
		public void VersionEqualsTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr2 = new EvrStringType (e2);
			EvrStringType evr3 = new EvrStringType (e3);
			EvrStringType evr4 = new EvrStringType (e4);			
			
			Assert.IsFalse ((evr1 == evr2), "evr 1:1.0.0-0ubuntu4 should not equal 1:1.0.0-0ubuntu3");
			Assert.IsFalse ((evr1 == evr4), "evr 1:1.0.0-0ubuntu4 should not equal 1.5.0-2dog5");
			Assert.IsTrue ((evr1 == evr3), "evr 1:1.0.0-0ubuntu4 should be equal to 1:1.0.0-0ubuntu4");
		}

		[Test]
		public void VersionNotEqualsTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr2 = new EvrStringType (e2);
			EvrStringType evr3 = new EvrStringType (e3);
		
			Assert.IsFalse ((evr1 != evr3), "evr 1:1.0.0-0ubuntu4 should be equal to 1:1.0.0-0ubuntu4");
			Assert.IsTrue ((evr1 != evr2), "evr 1:1.0.0-0ubuntu4 should not equal 1:1.0.0-0ubuntu3");				
		}
		
		[Test]
		public void VersionLessThanTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr2 = new EvrStringType (e2);
			EvrStringType evr3 = new EvrStringType (e3);
			EvrStringType evr4 = new EvrStringType (e4);
			EvrStringType evr5 = new EvrStringType (e5);
		
			Assert.IsTrue ((evr1 < evr4), "evr 1:1.0.0-0ubuntu4 should be less than 1.5.0-2dog5");
			Assert.IsTrue ((evr2 < evr3), "evr 1:1.0.0-0ubuntu3 should be less than 1:1.0.0-0ubuntu4");			
			Assert.IsFalse ((evr5 < evr3), "evr 1:1.0.0-0ubuntu40 should be greater than 1:1.0.0-0ubuntu4");
		}

		[Test]
		public void VersionLessThanOrEqualTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr2 = new EvrStringType (e2);
			EvrStringType evr3 = new EvrStringType (e3);
			EvrStringType evr5 = new EvrStringType (e5);
			
			Assert.IsTrue ((evr2 <= evr3), "evr 1:1.0.0-0ubuntu3 should be less than or equal to 1:1.0.0-0ubuntu4");
			Assert.IsTrue ((evr1 <= evr3), "evr 1:1.0.0-0ubuntu4 should be less than or equal to 1:1.0.0-0ubuntu4");	
			Assert.IsFalse ((evr5 <= evr3), "evr 1:1.0.0-0ubuntu40 should be greater than 1:1.0.0-0ubuntu4");
		}

		[Test]
		public void VersionGreaterThanTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr2 = new EvrStringType (e2);
			EvrStringType evr3 = new EvrStringType (e3);
			EvrStringType evr4 = new EvrStringType (e4);
			EvrStringType evr5 = new EvrStringType (e5);
		
			Assert.IsFalse ((evr2 > evr3), "evr 1:1.0.0-0ubuntu3 should be less than or equal to 1:1.0.0-0ubuntu4");
			Assert.IsTrue ((evr4 > evr1), "evr 1.5.0-2dog5 should be greater than 1:1.0.0-0ubuntu4");
			Assert.IsTrue ((evr5 > evr3), "evr 1:1.0.0-0ubuntu40 should be greater than 1:1.0.0-0ubuntu4");
		}
	
		[Test]
		public void VersionGreaterThanOrEqualTest ()
		{
			EvrStringType evr1 = new EvrStringType (e1);
			EvrStringType evr3 = new EvrStringType (e3);
			EvrStringType evr4 = new EvrStringType (e4);
		
			Assert.IsTrue ((evr1 >= evr3), "evr 1:1.0.0-0ubuntu4 should be greater than or equal to 1:1.0.0-0ubuntu4");
			Assert.IsTrue ((evr4 >= evr1), "evr 1.5.0-2dog5 should be greater than 1:1.0.0-0ubuntu4");
		}		
	}
}
