//
// FileMD5TestTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Independent;

namespace SussenTests.Oval.Independent
{
	[TestFixture]
	public class FileMD5TestTest
	{
		FileMD5Test ft;
		ObjectElement oe;

		[SetUp]
		public void SetUp()
		{
			ft = new FileMD5Test ();

			oe = new ObjectElement ("filemd5_object", "oval:org.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", ".", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "oval.v5.test.definitions.xml", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;
		}

		[TearDown]
		public void TearDown ()
		{
			ft = null;
			oe = null;
		}

		[Test]
		public void TestFileMD5Test ()
		{
			CollectedItem[] testResults;
			testResults = ft.Run (null, oe);

			Assert.AreEqual (1, testResults.Length, "No results found for filemd5 test");
			bool foundHash = false;
			
			CollectedItem c = testResults[0];
			foreach (EntityObject ie in c.Items) {			
				if (ie.Name == "md5") {
#if MONO_BUILD
					Assert.AreEqual ("6cb29a3a3d795c9fbdb12d2c216def87", ie.Value, "The md5sum doesn't match oval.v5.test.definitions.xml");
#else
					Assert.AreEqual("c0d4c989ce3414fbce19efee6d97ac31", ie.Value, "The md5sum doesn't match oval.v5.test.definitions.xml");
#endif

					foundHash = true;
				}
			}
			
			Assert.IsTrue (foundHash, "No md5 hash found in test results");
		}
	}
}
