//
// XmlFileContentTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Independent;

namespace SussenTests.Oval.Independent
{
	[TestFixture]
	public class XmlFileContentTestTest
	{
		XmlFileContentTest xt;
		ObjectElement oe;

		[SetUp]
		public void SetUp()
		{
			xt = new XmlFileContentTest ();

			oe = new ObjectElement ("xmlfilecontent_object", "oval:org.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", ".", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "xpath.xml", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			EntityObject eo3 = new EntityObject ("xpath", "//Employee", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;
		}

		[TearDown]
		public void TearDown ()
		{
			xt = null;
			oe = null;
		}

		[Test]
		public void TestXmlFileContentTest ()
		{
			CollectedItem[] testResults;
			testResults = xt.Run (null, oe);

			Assert.AreEqual (1, testResults.Length,  "No results found for xmlfilecontent test");
			CollectedItem c = testResults[0];
			
			bool goodResults = false;

			foreach (EntityObject ie in c.Items) {
				if (ie.Name == "value_of") {
					if (!ie.Value.StartsWith ("RustyShackleford"))
						Assert.Fail ("XmlFileContent test failed");
						
					goodResults = true;
				}
			}
			
			Assert.IsTrue (goodResults, "Failed to get results from xmlfilecontent test");
		}
	}
}
