//
// InterpreterTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.IO;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;

namespace SussenTests.Oval
{
	[TestFixture]
	public class InterpreterTest
	{
		Config config;

		[SetUp]
		public void SetUp()
		{
#if MONO_BUILD
			config = new Config ("unix");
            config.DefinitionsFile = "oval.v5.test.definitions.xml";
#else
            config = new Config("windows");
            config.DefinitionsFile = "windows.test.definitions.xml";
#endif			
			config.VerifyXML = false;
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
		}

		[Test]
		public void TestInterpreterTest ()
		{
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		public void TestInterpreterAnalyzeDefinition ()
		{
#if MONO_BUILD
			StreamReader sr = new StreamReader("definitionString.xml");
#else
            StreamReader sr = new StreamReader("windows.test.definitions.xml");
#endif
			string rawData = sr.ReadToEnd ();


			Interpreter i = new Interpreter ();
			i.CurrentConfiguration = config;
			string res = i.AnalyzeDefinition (rawData);

			Assert.IsNotNull (res, "AnalyzeDefinition call failed");
		}
	}
}
