//
// MultipleCriteriaTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;

#if MONO_BUILD
using Sussen.Oval.Unix;
#else
using Sussen.Oval.Windows;
#endif

namespace SussenTests.Oval
{
	[TestFixture]
	public class MultipleCriteriaTest
	{
		Config config;
		OvalDocuments docs;
		DataCollector dc;

		[SetUp]
		public void SetUp()
		{
#if MONO_BUILD
			config = new Config ("unix");
			config.DefinitionsFile = "MultipleCriteriaTest.xml";
#else
            config = new Config("windows");
            config.DefinitionsFile = "windows.multiple.criteria.test.xml";
#endif
			docs = new OvalDocuments ();
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
			docs = null;
		}

		[Test]
		public void TestMultipleCriteria ()
		{
			docs.Definitions.Load (config.DefinitionsFile);
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
#if MONO_BUILD
            docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());
#else
            docs.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:org.mitre.oval:def:2") {
					Assert.AreEqual ("true", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.AreEqual (true, allGood, "Test results not found");
		}
	}
}
