//
// UnixDpkgInfoTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class UnixDpkgInfoTest
	{
		DpkgInfoTest dt;
		ObjectElement oe;

		[SetUp]
		public void SetUp()
		{
			dt = new DpkgInfoTest ("sample.dpkg.status");
			oe = new ObjectElement ("dpkginfo_object", "oval:org.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "linux-image-2.6.15-25-386", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
		}

		[TearDown]
		public void TearDown ()
		{
			dt = null;
			oe = null;
		}

		[Test]
		public void TestDpkgInfoObject ()
		{
			CollectedItem[] testResults;
			testResults = dt.Run (null, oe);

			Assert.AreEqual (1, testResults.Length, "No results found for dpkginfo test");
			bool foundVersion = false;
			
			CollectedItem c = testResults[0];
			foreach (EntityObject ie in c.Items) {			
				if (ie.Name == "evr") {
					Assert.AreEqual ("2.6.15-25.43", ie.Value, "Package version doesn't match");
					foundVersion = true;
				}
			}
			
			Assert.IsTrue (foundVersion, "No package version found in test results");
		}
	}
}
