//
// InetListeningServersTestTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class InetListeningServersTestTest
	{
		InetListeningServersTest it;
		ObjectElement oe;
	
		Config config;
		DataCollector dc;
		OvalDocuments docs;

		[SetUp]
		public void SetUp()
		{
			config = new Config ("unix");
			config.DefinitionsFile = "inetlisteningservers.xml";
				
			docs = new OvalDocuments ();
			
			it = new InetListeningServersTest ();			
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;			
			docs = null;
			
			it = null;
			oe = null;
		}

		[Test]
		public void TestInetListeningServersObjectByPatternMatch ()
		{
			oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", ".*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("local_address", ".*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			
			
			EntityObject eo3 = new EntityObject ("local_port", ".*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;
			
			CollectedItem[] testResults;		
			testResults = it.Run (null, oe);

			Assert.IsFalse (testResults.Length > 500, 
				"Found large number of results (~{0}) in inetlisteningservers test; Possible broken test", testResults.Length);
		}

		[Test]
		public void TestInetListeningServersPatternMatch ()
		{
			docs.Definitions.Load (config.DefinitionsFile);
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
			docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity.oval:def:1") {
					Assert.AreEqual ("false", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.IsTrue (allGood, "No test results found");
		}

		[Test]
		public void TestInetListeningServersTest ()
		{
			docs.Definitions.Load (config.DefinitionsFile);
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
			docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity.oval:def:2") {
					Assert.AreEqual ("false", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.IsTrue (allGood, "No test results found");
		}		
	}
}
