//
// UnixInetdTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class UnixInetdTest
	{
		InetdTest it;
		ObjectElement oe;

		[SetUp]
		public void SetUp()
		{
			it = new InetdTest ("sample.inetd.conf");
			oe = new ObjectElement ("inetd_object", "oval:org.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("service_name", "finger", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;
		}

		[TearDown]
		public void TearDown ()
		{
			it = null;
			oe = null;
		}

		[Test]
		public void TestInetdTestObject ()
		{
			CollectedItem[] testResults;
			testResults = it.Run (null, oe);

			Assert.AreEqual (18, testResults.Length, "Incorrect number of results found in inetd_test");
			bool found = false;
			
			CollectedItem c = testResults[0];
			foreach (EntityObject ie in c.Items) {			
				if (ie.Name == "exec_as_user") {
					Assert.AreEqual ("root", ie.Value, "exec_as_user doesn't match, should be root");
					found = true;
				}
			}
			
			Assert.IsTrue (found, "No exec_as_user found in test results");
		}
	}
}
