//
// PasswordTestTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class PasswordTestTest
	{
		Config config;
		DataCollector dc;
		OvalDocuments docs;

		[SetUp]
		public void SetUp()
		{
			config = new Config ("unix");
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			
			docs = new OvalDocuments ();
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
			docs = null;
		}

		[Test]
		public void TestPasswordTest ()
		{
			docs.Definitions.Load (config.DefinitionsFile);
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
			docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:org.mmgsecurity:def:4") {
					Assert.AreEqual ("true", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.AreEqual (true, allGood, "No test results found");
		}
	}
}
