//
// Version.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using NUnit.Framework;
using Sussen.Oval;

namespace SussenTests.Oval
{
	[TestFixture]
	public class VersionTests
	{
		const string v1 = "1.0.0";
		const string v2 = "1.0.6";
		const string v3 = "1.0.4.1";
		const string v4 = "1-0-5-2";
		const string v5 = "0-2-0";
		const string v6 = "1-0-0";
		const string v7 = "6.00.3790.2759 (srv03_sp1_gdr.060727-2355)";
		const string v8 = "6.0.3790.2491";
		const string v9 = "5.2.3790.1830";
		const string v10 = "6.0.6603.0";
		const string v11 = "1";
	
		[Test]
		public void VersionEqualsTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver2 = new VersionType (v2);
			VersionType ver6 = new VersionType (v6);
			VersionType ver11 = new VersionType (v11);
		
			if (ver1 == ver2)
				Assert.Fail ("Version {0} should not equal {1}", v1, v2);
		
			Assert.IsTrue ((ver1 == ver6), "version numbers 1.0.0 / 1-0-0 should be equal");
			Assert.IsTrue ((ver1 == ver11), "version numbers 1.0.0 / 1 should be equal");
		}
		
		[Test]
		public void VersionNotEqualsTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver4 = new VersionType (v4);
			VersionType ver6 = new VersionType (v6);
		
			Assert.IsTrue ((ver1 != ver4), "version numbers 1.0.0 / 1-0-5-2 should be not equal");
			Assert.IsFalse ((ver1 != ver6), "version numbers 1.0.0 / 1-0-0 should be equal");				
		}
		
		[Test]
		public void VersionLessThanTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver2 = new VersionType (v2);
			VersionType ver6 = new VersionType (v6);

			VersionType ver7 = new VersionType (v7);
			VersionType ver8 = new VersionType (v8);
		
			Assert.IsTrue ((ver1 < ver2), "version numbers 1.0.0 / 1.0.6 should be less than");
			Assert.IsFalse ((ver1 < ver6), "version numbers 1.0.0 / 1-0-0 should be equal");
			
			Assert.IsFalse ((ver2 < ver1), "version numbers 1.0.6 / 1.0.0 should be greater than");

			Assert.IsFalse ((ver7 < ver8), "version 6.00.3790.2759 (srv03_sp1_gdr.060727-2355) should be greater than 6.0.3790.2491");
		}
		
		[Test]
		public void VersionLessThanOrEqualTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver2 = new VersionType (v2);
			VersionType ver6 = new VersionType (v6);
		
			Assert.IsTrue ((ver1 <= ver2), "version numbers 1.0.0 / 1.0.6 should be less than or equal");			
			Assert.IsTrue ((ver1 <= ver6), "version numbers 1.0.0 / 1-0-0 should be less than or equal");
			
			Assert.IsFalse ((ver2 <= ver1), "version numbers 1.0.6 / 1.0.0 should be greater than");
		}
		
		[Test]
		public void VersionGreaterThanTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver2 = new VersionType (v2);
			VersionType ver6 = new VersionType (v6);
			VersionType ver9 = new VersionType (v9);
			VersionType ver10 = new VersionType (v10);
		
			Assert.IsTrue ((ver2 > ver1), "version numbers 1.0.6 / 1.0.0 should be greater than");
			Assert.IsFalse ((ver9 > ver10), "version numbers 5.2.3790.1830 should be less than 6.0.6603.0");
			Assert.IsFalse ((ver1 > ver6), "version numbers 1.0.0 / 1-0-0 should be equal");			
		}

		[Test]
		public void VersionGreaterThanOrEqualTest ()
		{
			VersionType ver1 = new VersionType (v1);
			VersionType ver2 = new VersionType (v2);
			VersionType ver6 = new VersionType (v6);
		
			Assert.IsTrue ((ver2 >= ver1), "version numbers 1.0.6 / 1.0.0 should be greater than");			
			Assert.IsTrue ((ver1 >= ver6), "version numbers 1.0.0 / 1-0-0 should be equal");
			
			Assert.IsFalse ((ver1 >= ver2), "version numbers 1.0.0 / 1.0.6 should be less than");
		}		
	}
}
