// 
// AgentBLL.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.ComponentModel;
using System.Data;
using System.Configuration;
using Sussen.Web.Data;

namespace Sussen.Web.BusinessLogic
{
//    [DataObject]
    public class AgentBLL
    {
        AgentsTableAdapter adapter = null;
        AgentStatusTableAdapter statusAdapter = null;
        AgentPolicyTableAdapter policyAdapter = null;

//        [DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public AgentsDataTable GetAgents ()
        {
            return Adapter.GetAgents ();
        }

//        [DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public AgentsDataTable GetAgentsByAgentId(string agentID)
        {
            return Adapter.GetAgentById(agentID);
        }

//        [DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public AgentStatusDataTable GetAgentStatus(string agentID)
        {
            return StatusAdapter.GetAgentStatusById(agentID);
        }

//        [DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public string RegisterAgent(string hostName)
        {
        	if (hostName == null)
        		throw new ArgumentNullException ("hostName");
                	
        	string agentID = System.Guid.NewGuid().ToString(); 
			
			int retVal = Adapter.Insert (hostName, agentID);
        	if (retVal == 1) {
        		StatusAdapter.Insert (agentID);
        		return agentID;
        	}
        		
            return null;
        }

//        [DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public int DeleteAgent(string agentID)
        {
        	if (agentID == null)
        		throw new ArgumentNullException ("agentID");
        		
            return Adapter.Delete(agentID);
        }

		public string GetAgentPolicy (string agentID)
		{
        	if (agentID == null)
        		throw new ArgumentNullException ("agentID");
        		
        	AgentPolicyDataTable dt = PolicyAdapter.GetDataByAgentId (agentID);
        	if (dt.Count != 1)
        		return null;
        		
        	AgentPolicyRow row = dt[0];
        	return row.Definitions;
		}

        protected AgentsTableAdapter Adapter
        {
            get
            {
                if (adapter == null)
                    adapter = new AgentsTableAdapter();

                return adapter;
            }
        }

        protected AgentPolicyTableAdapter PolicyAdapter
        {
            get
            {
                if (policyAdapter == null)
                    policyAdapter = new AgentPolicyTableAdapter();

                return policyAdapter;
            }
        }
        
        protected AgentStatusTableAdapter StatusAdapter
        {
            get
            {
                if (statusAdapter == null)
                    statusAdapter = new AgentStatusTableAdapter();

                return statusAdapter;
            }
        }
    }
}
