// 
// ScanResultsBLL.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.ComponentModel;
using System.Data;
using System.Configuration;
using Sussen.Web.Data;

namespace Sussen.Web.BusinessLogic
{
//    [DataObject]
    public class ScanResultsBLL
    {
        AgentStatusTableAdapter agentAdapter = null;
        ScansTableAdapter scansAdapter = null;
        ScanResultsTableAdapter resultsAdapter = null;        
        DefinitionResultsTableAdapter defAdapter = null;

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public ScanResultsDataTable GetScanResults ()
        {
            return ResultsAdapter.GetData();
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public ScansDataTable GetScansByAgentId(string agentID)
        {
            return ScansAdapter.GetDataByAgentID(agentID);
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public ScanResultsDataTable GetScanResultsByAgentId (string agentID)
        {
            return ResultsAdapter.GetDataByAgentId(agentID);
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public ScanResultsDataTable GetScanResultsByScanId (int scanID)
        {
            return ResultsAdapter.GetDataByScanId(scanID);
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public DefinitionResultsDataTable GetVulnerabilitesByAgentId(string agentID)
        {
            return DefinitionAdapter.GetDataByAgentId(agentID);
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Select, true)]
        public DefinitionResultsDataTable SearchDefinitionResultsByReferenceId(string referenceID)
        {
            return DefinitionAdapter.GetDataByReferenceId(referenceID);
        }

        DateTime GetScanTime(string ovalTime)
        {
            DateTime scanTime = DateTime.Now;

            string tmp = ovalTime.Replace('T', ' ');
            scanTime = Convert.ToDateTime(tmp);

            return scanTime;
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Update, true)]
        public int InsertScanResults(string agentID, string xml)
        {
            int scanID = -1;

            try
            {
                scanID = ScansAdapter.Insert (agentID);               
                if (scanID < 0)
                    return scanID;

                ResultsAdapter.Insert(scanID, agentID, xml);

                OvalResultsData ord = new OvalResultsData(xml);

                DataTable genTable = ord.GeneratorDataSet.Tables["generator"];
                DataRow genRow = genTable.Rows[0];
                object productVersion = genRow["product_version"];
                object ts = genRow["timestamp"];
                DateTime scanTime = GetScanTime (ts.ToString());

                AgentStatusAdapter.Update (agentID, productVersion.ToString(), scanTime);

//                DataTable defTable = ord.DefinitionsDataSet.Tables["definition"];
//                foreach (DataRow row in defTable.Rows)
//                {                    
//                    object rowID = row["definition_id"];
//                    int rowNum = int.Parse(rowID.ToString());
//
//                    object id = row["id"];
//
//                    DataTable metaTable = ord.DefinitionsDataSet.Tables["metadata"];
//                    DataRow metaRow = metaTable.Rows[rowNum];
//                    object title = metaRow["title"];
//                    object description = metaRow["description"];
//
//                    DataTable referenceTable = ord.DefinitionsDataSet.Tables["reference"];
//                    DataRow referenceRow = referenceTable.Rows[rowNum];
//                    object refSource = referenceRow["source"];
//                    object refId = referenceRow["ref_id"];
//                    object refUrl = referenceRow["ref_url"];
//
//                    DataTable affectedTable = ord.DefinitionsDataSet.Tables["affected"];
//                    DataRow affectedRow = affectedTable.Rows[rowNum];
//                    object affFamily = affectedRow["family"];
//                    object affPlatform = affectedRow["platform"];
//                    object affProduct = affectedRow["product"];
//
//                    DataTable dTable = ord.ResultsDataSet.Tables["definition"];
//                    DataRow dRow = dTable.Rows[rowNum];
//                    object result = dRow["result"];
//                    
//                    bool isssueResolved = true;
//                    if (result.ToString().ToLower() == "true")
//                        isssueResolved = false;
//
//                    DefinitionAdapter.Insert(scanID, agentID, id.ToString(), title.ToString(),
//                        description.ToString(), refSource.ToString(), refId.ToString(),
//                        refUrl.ToString(), affFamily.ToString(), affPlatform.ToString(),
//                        affProduct.ToString(), result.ToString(), isssueResolved);
//                }

            } catch (Exception e) {
				Console.WriteLine (e.ToString());            
            }

            return scanID;
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Update, true)]
        public int DeleteScanResults(int scanID)
        {
            return ScansAdapter.Delete(scanID);
        }

//        [DataObjectMethodAttribute(DataObjectMethodType.Update, true)]
        public int DeleteScanResultsByAgentId(string agentID)
        {
            return ResultsAdapter.DeleteScanResultsByAgentId(agentID);
        }

        protected AgentStatusTableAdapter AgentStatusAdapter
        {
            get
            {
                if (agentAdapter == null)
                    agentAdapter = new AgentStatusTableAdapter();

                return agentAdapter;
            }
        }

        protected DefinitionResultsTableAdapter DefinitionAdapter
        {
            get
            {
                if (defAdapter == null)
                    defAdapter = new DefinitionResultsTableAdapter();

                return defAdapter;
            }
        }

        protected ScanResultsTableAdapter ResultsAdapter
        {
            get
            {
                if (resultsAdapter == null)
                    resultsAdapter = new ScanResultsTableAdapter();

                return resultsAdapter;
            }
        }

        protected ScansTableAdapter ScansAdapter
        {
            get
            {
                if (scansAdapter == null)
                    scansAdapter = new ScansTableAdapter();

                return scansAdapter;
            }
        }
    }
}
