// 
// sussen-web - AgentPolicyDataTable.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Data;
using System.Runtime.Serialization;
using System.Xml.Schema;

namespace Sussen.Web.Data
{
	[System.Serializable()]
//	[System.Xml.Serialization.XmlSchemaProviderAttribute("GetTypedTableSchema")]
	public class AgentPolicyDataTable : DataTable, IEnumerable
	{
		DataColumn colAgentId;
		DataColumn colDefinitions;
		
		public AgentPolicyDataTable () 
		{
			this.TableName = "agent_policy";
			this.BeginInit ();
			this.InitClass ();
			this.EndInit ();
		}
			
		protected AgentPolicyDataTable (SerializationInfo info, StreamingContext context) : base (info, context)
		{
			InitVars ();
		}
		
		internal AgentPolicyDataTable (DataTable table)
		{
			this.TableName = table.TableName;
			
			if (table.CaseSensitive != table.DataSet.CaseSensitive)
				this.CaseSensitive = table.CaseSensitive;
				
			if (table.Locale.ToString() != table.DataSet.Locale.ToString())
				this.Locale = table.Locale;
				
			if (table.Namespace != table.DataSet.Namespace)
				this.Namespace = table.Namespace;
				
			this.Prefix = table.Prefix;
			this.MinimumCapacity = table.MinimumCapacity;
		}

		public virtual System.Collections.IEnumerator GetEnumerator() 
		{
			return this.Rows.GetEnumerator();
		}

		public int Count
		{
			get { return this.Rows.Count; }
		}
		
		public DataColumn AgentIdColumn
		{
			get { return colAgentId; }
		}
		
		public DataColumn DefinitionsColumn
		{
			get { return colDefinitions; }
		}
		
		public AgentPolicyRow this [int index]
		{
			get { return (AgentPolicyRow) this.Rows [index]; }
		}
		
		void InitClass()
		{
			colAgentId = new DataColumn ("agent_id", typeof(string), null, System.Data.MappingType.Element);
			colAgentId.AllowDBNull = false;
			colAgentId.MaxLength = 150;
			base.Columns.Add (colAgentId);
			
			colDefinitions = new DataColumn ("definitions", typeof(string), null, System.Data.MappingType.Element);
			colDefinitions.AllowDBNull = false;
			colDefinitions.MaxLength = 255;
			base.Columns.Add (colDefinitions);			
		}
		
		public void InitVars ()
		{
			colAgentId = base.Columns["agent_id"];
			colDefinitions = base.Columns["definitions"];
		}
	}
}