// 
// sussen-web - AgentPolicyTableAdapter.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace Sussen.Web.Data
{
	public class AgentPolicyTableAdapter
	{
		MySql.Data.MySqlClient.MySqlDataAdapter adapter;
		MySql.Data.MySqlClient.MySqlConnection connection;
		MySql.Data.MySqlClient.MySqlCommand[] commandCollection;
		bool clearBeforeFill;

		public AgentPolicyTableAdapter ()
		{
			clearBeforeFill = true;
		}

		public bool ClearBeforeFill 
		{
            get { return clearBeforeFill; }
            set { this.clearBeforeFill = value; }
        }

		public virtual int DeleteAgentPolicyByAgentId (string agentID) 
		{
			if (agentID == null)
				throw new ArgumentNullException ("agentID");
				
			MySql.Data.MySqlClient.MySqlCommand command = this.CommandCollection[1];
			command.Parameters[0].Value = agentID;
			command.Connection.Open();		

			int retVal;
			try {
				retVal = command.ExecuteNonQuery();
			} finally { 
				command.Connection.Close();
			}
	
			return retVal;
		}

		public virtual int Fill (AgentPolicyDataTable dataTable) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[0];
			if (clearBeforeFill)
				dataTable.Clear();
			
			int returnValue = this.Adapter.Fill(dataTable);
			return returnValue;
		}

		public virtual int FillByAgentId (AgentPolicyDataTable dataTable, string agentID) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[2];
			this.Adapter.SelectCommand.Parameters[0].Value = agentID;
			
			if (clearBeforeFill)
				dataTable.Clear();

			int returnValue = this.Adapter.Fill(dataTable);
			return returnValue;
		}

		public virtual AgentPolicyDataTable GetData() 
		{
			this.Adapter.SelectCommand = this.CommandCollection[0];
			AgentPolicyDataTable dataTable = new AgentPolicyDataTable();
			this.Adapter.Fill(dataTable);
			return dataTable;
		}

		public virtual AgentPolicyDataTable GetDataByAgentId (string agentID) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[2];
			this.Adapter.SelectCommand.Parameters[0].Value = agentID;
			AgentPolicyDataTable dataTable = new AgentPolicyDataTable();
			this.Adapter.Fill(dataTable);
			return dataTable;
		}

		public virtual int Insert (string agentID, string definitions)
		{
			if (agentID == null)
				throw new ArgumentNullException ("agentID");
		
			this.Adapter.InsertCommand.Parameters[0].Value = agentID;
			this.Adapter.InsertCommand.Parameters[1].Value = definitions;
			
			if ((this.Adapter.InsertCommand.Connection.State & ConnectionState.Open) != ConnectionState.Open)
				this.Adapter.InsertCommand.Connection.Open ();

			try {
			
				int retVal = this.Adapter.InsertCommand.ExecuteNonQuery ();
				return retVal;
				
			} finally {
				this.Adapter.InsertCommand.Connection.Close ();
			}
		}
		
		void InitAdapter ()
		{
			adapter = new MySql.Data.MySqlClient.MySqlDataAdapter();
			
			DataTableMapping tableMapping = new DataTableMapping();
			tableMapping.SourceTable = "Table";
			tableMapping.DataSetTable = "agent_policy";
			tableMapping.ColumnMappings.Add("agent_id", "agent_id");
			tableMapping.ColumnMappings.Add("definitions", "definitions");

			adapter.TableMappings.Add(tableMapping);

			adapter.InsertCommand = new MySql.Data.MySqlClient.MySqlCommand();
			adapter.InsertCommand.CommandText = "INSERT INTO agent_policy VALUES (?agent_id, ?definitions)";
			adapter.InsertCommand.Connection = this.Connection;
			adapter.InsertCommand.Parameters.Add ("agent_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("definitions", MySql.Data.MySqlClient.MySqlDbType.VarChar);
		}
		
		void InitCommandCollection() 
		{
            this.commandCollection = new MySql.Data.MySqlClient.MySqlCommand[4];
            
            this.commandCollection[0] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[0].Connection = this.Connection;
            this.commandCollection[0].CommandText = "SELECT * FROM agent_policy";
            this.commandCollection[0].CommandType = System.Data.CommandType.Text;
            
            this.commandCollection[1] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[1].Connection = this.Connection;
            this.commandCollection[1].CommandText = "DELETE FROM agent_policy WHERE agent_id=?agent_id";
            this.commandCollection[1].CommandType = System.Data.CommandType.Text;
            this.commandCollection[1].Parameters.Add("agent_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
            
            this.commandCollection[2] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[2].Connection = this.Connection;
            this.commandCollection[2].CommandText = "SELECT * FROM agent_policy WHERE agent_id=?agent_id";
            this.commandCollection[2].CommandType = System.Data.CommandType.Text;
            this.commandCollection[2].Parameters.Add("agent_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);            
        }
		
		void InitConnection()
		{
			ConnectionStringSettings cs = ConfigurationManager.ConnectionStrings ["SussenWeb"];
			
			connection = new MySql.Data.MySqlClient.MySqlConnection();
			connection.ConnectionString = cs.ConnectionString;	
		}
		
		MySql.Data.MySqlClient.MySqlDataAdapter Adapter 
		{
            get {
            	if (this.adapter == null)
            		this.InitAdapter ();

                return this.adapter;
            }
        }
        
        MySql.Data.MySqlClient.MySqlConnection Connection 
        {
            get {     
            	if (this.connection == null)
            		this.InitConnection();

                return this.connection;
            }
            
            set {
            
                this.connection = value;
                if (this.Adapter.InsertCommand != null)
                    this.Adapter.InsertCommand.Connection = value;

                if (this.Adapter.DeleteCommand != null)
                    this.Adapter.DeleteCommand.Connection = value;
                
                if (this.Adapter.UpdateCommand != null)
                    this.Adapter.UpdateCommand.Connection = value;
                
                for (int i = 0; (i < this.CommandCollection.Length); i = (i + 1)) {
                    if ((this.CommandCollection[i] != null)) {
                        ((MySql.Data.MySqlClient.MySqlCommand)(this.CommandCollection[i])).Connection = value;
                    }
                }
            }
		}
		
		protected MySql.Data.MySqlClient.MySqlCommand[] CommandCollection 
		{
            get {
                if (this.commandCollection == null)
                    this.InitCommandCollection();

                return this.commandCollection;
            }
        }     
	}
}
