// 
// sussen-web - AgentStatusDataTable.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Data;
using System.Runtime.Serialization;
using System.Xml.Schema;

namespace Sussen.Web.Data
{
	[System.Serializable()]
//	[System.Xml.Serialization.XmlSchemaProviderAttribute("GetTypedTableSchema")]
	public class AgentStatusDataTable : DataTable, IEnumerable
	{
		DataColumn colId;
		DataColumn colVersion;
		DataColumn colLastScan;
		
		public AgentStatusDataTable () 
		{
			this.TableName = "agent_status";
			this.BeginInit ();
			this.InitClass ();
			this.EndInit ();
		}
			
		protected AgentStatusDataTable (SerializationInfo info, StreamingContext context) : base (info, context)
		{
			InitVars ();
		}
		
		internal AgentStatusDataTable (DataTable table)
		{
			this.TableName = table.TableName;
			
			if (table.CaseSensitive != table.DataSet.CaseSensitive)
				this.CaseSensitive = table.CaseSensitive;
				
			if (table.Locale.ToString() != table.DataSet.Locale.ToString())
				this.Locale = table.Locale;
				
			if (table.Namespace != table.DataSet.Namespace)
				this.Namespace = table.Namespace;
				
			this.Prefix = table.Prefix;
			this.MinimumCapacity = table.MinimumCapacity;
		}

//		public static XmlSchemaComplexType GetTypedTableSchema (XmlSchemaSet xs)
//		{
//			XmlSchemaComplexType type = new XmlSchemaComplexType();
//			XmlSchemaSequence sequence = new XmlSchemaSequence();
//		
//			SussenDataSet ds = new SussenDataSet();
//			xs.Add(ds.GetSchemaSerializable());
//			
//			XmlSchemaAny any1 = new XmlSchemaAny();
//			any1.Namespace = "http://www.w3.org/2001/XMLSchema";
//			any1.MinOccurs = new decimal(0);
//			any1.MaxOccurs = decimal.MaxValue;
//			any1.ProcessContents = XmlSchemaContentProcessing.Lax;
//			sequence.Items.Add(any1);
//			
//			XmlSchemaAny any2 = new XmlSchemaAny();
//			any2.Namespace = "urn:schemas-microsoft-com:xml-diffgram-v1";
//			any2.MinOccurs = new decimal(1);
//			any2.ProcessContents = System.Xml.Schema.XmlSchemaContentProcessing.Lax;
//			sequence.Items.Add(any2);
//			
//			XmlSchemaAttribute attribute1 = new XmlSchemaAttribute();
//			attribute1.Name = "namespace";
//			attribute1.FixedValue = ds.Namespace;
//			type.Attributes.Add(attribute1);
//			
//			XmlSchemaAttribute attribute2 = new XmlSchemaAttribute();
//			attribute2.Name = "tableTypeName";
//			attribute2.FixedValue = "agentsDataTable";
//			type.Attributes.Add(attribute2);
//			type.Particle = sequence;
//			
//			return type;
//		}

		public virtual System.Collections.IEnumerator GetEnumerator() 
		{
			return this.Rows.GetEnumerator();
		}

		public int Count
		{
			get { return this.Rows.Count; }
		}
		
		public DataColumn IdColumn
		{
			get { return colId; }
		}
		
		public DataColumn VersionColumn
		{
			get { return colVersion; }
		}
		
		public DataColumn LastScanColumn
		{
			get { return colLastScan; }
		}
		
		public AgentStatusRow this [int index]
		{
			get { return (AgentStatusRow) this.Rows [index]; }
		}
		
		void InitClass()
		{
			colId = new DataColumn ("agent_id", typeof(string), null, System.Data.MappingType.Element);
			colId.AllowDBNull = false;
			colId.Unique = true;
			colId.MaxLength = 150;
			base.Columns.Add (colId);
			
			colVersion = new DataColumn ("product_version", typeof(string), null, System.Data.MappingType.Element);
			colVersion.MaxLength = 10;
			base.Columns.Add (colVersion);
			
			colLastScan = new DataColumn ("last_scan", typeof(System.DateTime), null, System.Data.MappingType.Element);
			base.Columns.Add (colLastScan);			
		}
		
		public void InitVars ()
		{
			colId = base.Columns["agent_id"];
			colVersion = base.Columns["product_version"];
			colLastScan = base.Columns["last_scan"];
		}
	}
}