// 
// sussen-web - AgentsTableAdapter.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace Sussen.Web.Data
{
	public class AgentsTableAdapter
	{
		MySql.Data.MySqlClient.MySqlDataAdapter adapter;
		MySql.Data.MySqlClient.MySqlConnection connection;
		MySql.Data.MySqlClient.MySqlCommand[] commandCollection;
		bool clearBeforeFill;

		public AgentsTableAdapter ()
		{
			clearBeforeFill = true;
		}

		public bool ClearBeforeFill 
		{
            get { return clearBeforeFill; }
            set { this.clearBeforeFill = value; }
        }

		public virtual int Delete(string agentID) 
		{
			if (agentID == null)
				throw new ArgumentNullException ("agentID");
				
            this.Adapter.DeleteCommand.Parameters[0].Value = agentID;
            
            if ((this.Adapter.DeleteCommand.Connection.State & ConnectionState.Open) != ConnectionState.Open)
				this.Adapter.DeleteCommand.Connection.Open ();
            
            try {
            
                int returnValue = this.Adapter.DeleteCommand.ExecuteNonQuery();
                return returnValue;
                
            } finally {
				this.Adapter.DeleteCommand.Connection.Close();
            }
        }

		public virtual int FillAgents(AgentsDataTable dataTable) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[0];
			if (clearBeforeFill)
				dataTable.Clear ();
			
            int returnValue = this.Adapter.Fill(dataTable);
            return returnValue;
        }

		public virtual int FillByAgentId(AgentsDataTable dataTable, string id) 
		{
            this.Adapter.SelectCommand = this.CommandCollection[1];
            this.Adapter.SelectCommand.Parameters[0].Value = (string) id;
            
            if (clearBeforeFill)
                dataTable.Clear();
            
            int returnValue = this.Adapter.Fill(dataTable);
            return returnValue;
        }

		public virtual AgentsDataTable GetAgents() 
		{
			this.Adapter.SelectCommand = this.CommandCollection[0];
			AgentsDataTable dataTable = new AgentsDataTable();
			this.Adapter.Fill(dataTable);
			return dataTable;
        }

		public virtual AgentsDataTable GetAgentById(string id) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[1];
			this.Adapter.SelectCommand.Parameters[0].Value = (string)id;
			AgentsDataTable dataTable = new AgentsDataTable();
			this.Adapter.Fill(dataTable);
			return dataTable;
        }

		public virtual int Insert (string hostName, string agentID)
		{
			if (hostName == null)
				throw new ArgumentNullException ("hostName");
		
			this.Adapter.InsertCommand.Parameters[0].Value = agentID;
			this.Adapter.InsertCommand.Parameters[1].Value = hostName;
			
			if ((this.Adapter.InsertCommand.Connection.State & ConnectionState.Open) != ConnectionState.Open)
				this.Adapter.InsertCommand.Connection.Open ();

			try {
			
				int retVal = this.Adapter.InsertCommand.ExecuteNonQuery ();
				return retVal;
				
			} finally {
				this.Adapter.InsertCommand.Connection.Close ();
			}
		}
		
		void InitAdapter ()
		{
			adapter = new MySql.Data.MySqlClient.MySqlDataAdapter();
			
			DataTableMapping tableMapping = new DataTableMapping();
			tableMapping.SourceTable = "Table";
			tableMapping.DataSetTable = "agents";
			tableMapping.ColumnMappings.Add("id", "id");
			tableMapping.ColumnMappings.Add("name", "name");

			adapter.TableMappings.Add(tableMapping);

			adapter.InsertCommand = new MySql.Data.MySqlClient.MySqlCommand();
			adapter.InsertCommand.CommandText = "INSERT INTO agents VALUES (?id, ?name)";
			adapter.InsertCommand.Connection = this.Connection;
			adapter.InsertCommand.Parameters.Add ("id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("name", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			
			adapter.DeleteCommand = new MySql.Data.MySqlClient.MySqlCommand();
			adapter.DeleteCommand.CommandText = "DELETE FROM agents WHERE id=?id";
			adapter.DeleteCommand.Connection = this.Connection;
			adapter.DeleteCommand.Parameters.Add ("id", MySql.Data.MySqlClient.MySqlDbType.VarChar);		
		}
		
		void InitCommandCollection() 
		{
            this.commandCollection = new MySql.Data.MySqlClient.MySqlCommand[2];
            
            this.commandCollection[0] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[0].Connection = this.Connection;
            this.commandCollection[0].CommandText = "SELECT id, name FROM agents";
            this.commandCollection[0].CommandType = System.Data.CommandType.Text;
            
            this.commandCollection[1] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[1].Connection = this.Connection;
            this.commandCollection[1].CommandText = "SELECT id, name FROM agents WHERE id=?id";
            this.commandCollection[1].CommandType = System.Data.CommandType.Text;
            this.commandCollection[1].Parameters.Add("id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
        }
		
		void InitConnection()
		{
			ConnectionStringSettings cs = ConfigurationManager.ConnectionStrings ["SussenWeb"];
			
			connection = new MySql.Data.MySqlClient.MySqlConnection();
			connection.ConnectionString = cs.ConnectionString;	
		}
		
		MySql.Data.MySqlClient.MySqlDataAdapter Adapter 
		{
            get {
            	if (this.adapter == null)
            		this.InitAdapter ();

                return this.adapter;
            }
        }
        
        MySql.Data.MySqlClient.MySqlConnection Connection 
        {
            get {     
            	if (this.connection == null)
            		this.InitConnection();

                return this.connection;
            }
            
            set {
            
                this.connection = value;
                if (this.Adapter.InsertCommand != null)
                    this.Adapter.InsertCommand.Connection = value;

                if (this.Adapter.DeleteCommand != null)
                    this.Adapter.DeleteCommand.Connection = value;
                
                if (this.Adapter.UpdateCommand != null)
                    this.Adapter.UpdateCommand.Connection = value;
                
                for (int i = 0; (i < this.CommandCollection.Length); i = (i + 1)) {
                    if ((this.CommandCollection[i] != null)) {
                        ((MySql.Data.MySqlClient.MySqlCommand)(this.CommandCollection[i])).Connection = value;
                    }
                }
            }
		}
		
		protected MySql.Data.MySqlClient.MySqlCommand[] CommandCollection 
		{
            get {
                if (this.commandCollection == null)
                    this.InitCommandCollection();

                return this.commandCollection;
            }
        }     
	}
}
