// 
// sussen-web - DefinitionResultsDataTable.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Data;
using System.Runtime.Serialization;
using System.Xml.Schema;

namespace Sussen.Web.Data
{
	[System.Serializable()]
//	[System.Xml.Serialization.XmlSchemaProviderAttribute("GetTypedTableSchema")]
	public class DefinitionResultsDataTable : DataTable, IEnumerable
	{
		DataColumn colId;
		DataColumn colAgentId;
		DataColumn colDefinitionId;
		DataColumn colTitle;
		DataColumn colDescription;
		DataColumn colRefSource;
		DataColumn colRefId;
		DataColumn colRefUrl;
		DataColumn colAffectedFamily;
		DataColumn colAffectedPlatform;
		DataColumn colAffectedProduct;
		DataColumn colResult;
		DataColumn colScanId;
		DataColumn colIssueResolved;
		
		public DefinitionResultsDataTable () 
		{
			this.TableName = "definition_results";
			this.BeginInit ();
			this.InitClass ();
			this.EndInit ();
		}
			
		protected DefinitionResultsDataTable (SerializationInfo info, StreamingContext context) : base (info, context)
		{
			InitVars ();
		}
		
		internal DefinitionResultsDataTable (DataTable table)
		{
			this.TableName = table.TableName;
			
			if (table.CaseSensitive != table.DataSet.CaseSensitive)
				this.CaseSensitive = table.CaseSensitive;
				
			if (table.Locale.ToString() != table.DataSet.Locale.ToString())
				this.Locale = table.Locale;
				
			if (table.Namespace != table.DataSet.Namespace)
				this.Namespace = table.Namespace;
				
			this.Prefix = table.Prefix;
			this.MinimumCapacity = table.MinimumCapacity;
		}

//		public static XmlSchemaComplexType GetTypedTableSchema (XmlSchemaSet xs)
//		{
//			XmlSchemaComplexType type = new XmlSchemaComplexType();
//			XmlSchemaSequence sequence = new XmlSchemaSequence();
//		
//			SussenDataSet ds = new SussenDataSet();
//			xs.Add(ds.GetSchemaSerializable());
//			
//			XmlSchemaAny any1 = new XmlSchemaAny();
//			any1.Namespace = "http://www.w3.org/2001/XMLSchema";
//			any1.MinOccurs = new decimal(0);
//			any1.MaxOccurs = decimal.MaxValue;
//			any1.ProcessContents = XmlSchemaContentProcessing.Lax;
//			sequence.Items.Add(any1);
//			
//			XmlSchemaAny any2 = new XmlSchemaAny();
//			any2.Namespace = "urn:schemas-microsoft-com:xml-diffgram-v1";
//			any2.MinOccurs = new decimal(1);
//			any2.ProcessContents = System.Xml.Schema.XmlSchemaContentProcessing.Lax;
//			sequence.Items.Add(any2);
//			
//			XmlSchemaAttribute attribute1 = new XmlSchemaAttribute();
//			attribute1.Name = "namespace";
//			attribute1.FixedValue = ds.Namespace;
//			type.Attributes.Add(attribute1);
//			
//			XmlSchemaAttribute attribute2 = new XmlSchemaAttribute();
//			attribute2.Name = "tableTypeName";
//			attribute2.FixedValue = "agentsDataTable";
//			type.Attributes.Add(attribute2);
//			type.Particle = sequence;
//			
//			return type;
//		}

		public virtual System.Collections.IEnumerator GetEnumerator() 
		{
			return this.Rows.GetEnumerator();
		}

		public int Count
		{
			get { return this.Rows.Count; }
		}
		
		public DataColumn IdColumn
		{
			get { return colId; }
		}
		
		public DataColumn AgentIdColumn
		{
			get { return colAgentId; }
		}

		public DataColumn DefinitionIdColumn
		{
			get { return colDefinitionId; }
		}

		public DataColumn TitleColumn
		{
			get { return colTitle; }
		}
		
		public DataColumn DescriptionColumn
		{
			get { return colDescription; }
		}
		
		public DataColumn RefSourceColumn
		{
			get { return colRefSource; }
		}

		public DataColumn RefIdColumn
		{
			get { return colRefId; }
		}		
		
		public DataColumn RefUrlColumn
		{
			get { return colRefUrl; }
		}

		public DataColumn AffectedFamilyColumn
		{
			get { return colAffectedFamily; }
		}		

		public DataColumn AffectedPlatformColumn
		{
			get { return colAffectedPlatform; }
		}

		public DataColumn AffectedProductColumn
		{
			get { return colAffectedProduct; }
		}

		public DataColumn ResultColumn
		{
			get { return colResult; }
		}
		
		public DataColumn ScanIdColumn
		{
			get { return colScanId; }
		}
		
		public DataColumn IssueResolvedColumn
		{
			get { return colIssueResolved; }
		}		

		public DefinitionResultsRow this [int index]
		{
			get { return (DefinitionResultsRow) this.Rows [index]; }
		}
		
		void InitClass()
		{
			colId = new DataColumn ("id", typeof(int), null, System.Data.MappingType.Element);
			colId.AllowDBNull = false;
			colId.Unique = true;
			base.Columns.Add (colId);
			
			colScanId = new DataColumn ("scan_id", typeof(int), null, System.Data.MappingType.Element);
			colScanId.AllowDBNull = false;
			base.Columns.Add (colScanId);

			colAgentId = new DataColumn ("agent_id", typeof(string), null, System.Data.MappingType.Element);
			colAgentId.MaxLength = 150;
			colAgentId.AllowDBNull = false;
			base.Columns.Add (colAgentId);

			colDefinitionId = new DataColumn ("definition_id", typeof(string), null, System.Data.MappingType.Element);
			colDefinitionId.MaxLength = 255;
			colDefinitionId.AllowDBNull = false;
			base.Columns.Add (colDefinitionId);
			
			colTitle = new DataColumn ("title", typeof(string), null, System.Data.MappingType.Element);
			colTitle.MaxLength = 255;
			base.Columns.Add (colTitle);			

			colDescription = new DataColumn ("description", typeof(string), null, System.Data.MappingType.Element);
			base.Columns.Add (colDescription);
			
			colRefSource = new DataColumn ("ref_source", typeof(string), null, System.Data.MappingType.Element);
			colRefSource.MaxLength = 255;
			base.Columns.Add (colRefSource);
			
			colRefId = new DataColumn ("ref_id", typeof(string), null, System.Data.MappingType.Element);
			colRefId.MaxLength = 255;
			base.Columns.Add (colRefId);
			
			colRefUrl = new DataColumn ("ref_url", typeof(string), null, System.Data.MappingType.Element);
			colRefUrl.MaxLength = 255;
			base.Columns.Add (colRefUrl);			

			colAffectedFamily = new DataColumn ("affected_family", typeof(string), null, System.Data.MappingType.Element);
			colAffectedFamily.MaxLength = 25;
			base.Columns.Add (colAffectedFamily);
			
			colAffectedPlatform = new DataColumn ("affected_platform", typeof(string), null, System.Data.MappingType.Element);
			colAffectedPlatform.MaxLength = 255;
			base.Columns.Add (colAffectedPlatform);
			
			colAffectedProduct = new DataColumn ("affected_product", typeof(string), null, System.Data.MappingType.Element);
			colAffectedProduct.MaxLength = 255;
			base.Columns.Add (colAffectedProduct);			

			colResult = new DataColumn ("result", typeof(string), null, System.Data.MappingType.Element);
			colResult.MaxLength = 25;
			base.Columns.Add (colResult);

			colIssueResolved = new DataColumn ("issue_resolved", typeof(bool), null, System.Data.MappingType.Element);
			base.Columns.Add (colIssueResolved);
		}
		
		public void InitVars ()
		{
			colId = base.Columns["id"];
			colAgentId = base.Columns["agent_id"];
			colDefinitionId = base.Columns["definition_id"];
			colTitle = base.Columns["title"];
			colDescription = base.Columns["description"];
			colRefSource = base.Columns["ref_source"];
			colRefId = base.Columns["ref_id"];
			colRefUrl = base.Columns["ref_url"];
			colAffectedFamily = base.Columns["affected_family"];
			colAffectedPlatform = base.Columns["affected_platform"];
			colAffectedProduct = base.Columns["affected_product"];
			colResult = base.Columns["result"];
			colScanId = base.Columns["scan_id"];
			colIssueResolved = base.Columns["issue_resolved"];
		}
	}
}