// 
// sussen-web - DefinitionResultsTableAdapter.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace Sussen.Web.Data
{
	public class DefinitionResultsTableAdapter
	{
		MySql.Data.MySqlClient.MySqlDataAdapter adapter;
		MySql.Data.MySqlClient.MySqlConnection connection;
		MySql.Data.MySqlClient.MySqlCommand[] commandCollection;
		bool clearBeforeFill;

		public DefinitionResultsTableAdapter ()
		{
			clearBeforeFill = true;
		}

		public bool ClearBeforeFill 
		{
            get { return clearBeforeFill; }
            set { this.clearBeforeFill = value; }
        }

		public virtual int Insert (int scan_id, string agent_id, string definition_id, string title, string description, string ref_source, string ref_id, string ref_url, string affected_family, string affected_platform, string affected_product, string result, System.Nullable<bool> issue_resolved)
		{
			// FIXME: check arguments
			this.Adapter.InsertCommand.Parameters[0].Value = agent_id;
			this.Adapter.InsertCommand.Parameters[1].Value = definition_id;
			this.Adapter.InsertCommand.Parameters[2].Value = title;
			this.Adapter.InsertCommand.Parameters[3].Value = description;
			this.Adapter.InsertCommand.Parameters[4].Value = ref_source;
			this.Adapter.InsertCommand.Parameters[5].Value = ref_id;
			this.Adapter.InsertCommand.Parameters[6].Value = ref_url;
			this.Adapter.InsertCommand.Parameters[7].Value = affected_family;
			this.Adapter.InsertCommand.Parameters[8].Value = affected_platform;
			this.Adapter.InsertCommand.Parameters[9].Value = affected_product;
			this.Adapter.InsertCommand.Parameters[10].Value = result;
			this.Adapter.InsertCommand.Parameters[11].Value = scan_id;
			this.Adapter.InsertCommand.Parameters[12].Value = issue_resolved;
		
			if ((this.Adapter.InsertCommand.Connection.State & ConnectionState.Open) != ConnectionState.Open)
				this.Adapter.InsertCommand.Connection.Open ();

			try {
			
				int retVal = this.Adapter.InsertCommand.ExecuteNonQuery ();
				return retVal;
				
			} finally {
				this.Adapter.InsertCommand.Connection.Close ();
			}
		}
		
		public virtual DefinitionResultsDataTable GetDataByAgentId (string agentID) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[1];
			this.Adapter.SelectCommand.Parameters[0].Value = agentID;
			DefinitionResultsDataTable dataTable = new DefinitionResultsDataTable();
			this.Adapter.Fill(dataTable);
			return dataTable;
        }

		public virtual DefinitionResultsDataTable GetDataByReferenceId(string refID) 
		{
			this.Adapter.SelectCommand = this.CommandCollection[2];
			
			if (refID == null)
				this.Adapter.SelectCommand.Parameters[0].Value = System.DBNull.Value;
			else				
                this.Adapter.SelectCommand.Parameters[0].Value = refID;
            
            DefinitionResultsDataTable dataTable = new DefinitionResultsDataTable();
            this.Adapter.Fill(dataTable);
            
            return dataTable;
        }
		
		void InitAdapter ()
		{
			adapter = new MySql.Data.MySqlClient.MySqlDataAdapter();
			
			DataTableMapping tableMapping = new DataTableMapping();
			tableMapping.SourceTable = "Table";
			tableMapping.DataSetTable = "definition_results";
			tableMapping.ColumnMappings.Add("id", "id");
			tableMapping.ColumnMappings.Add("agent_id", "agent_id");
			tableMapping.ColumnMappings.Add("definition_id", "definition_id");
			tableMapping.ColumnMappings.Add("title", "title");
			tableMapping.ColumnMappings.Add("description", "description");
			tableMapping.ColumnMappings.Add("ref_source", "ref_source");
			tableMapping.ColumnMappings.Add("ref_id", "ref_id");
			tableMapping.ColumnMappings.Add("ref_url", "ref_url");
			tableMapping.ColumnMappings.Add("affected_family", "affected_family");
			tableMapping.ColumnMappings.Add("affected_platform", "affected_platform");
			tableMapping.ColumnMappings.Add("affected_product", "affected_product");
			tableMapping.ColumnMappings.Add("result", "result");
			tableMapping.ColumnMappings.Add("scan_id", "scan_id");
			tableMapping.ColumnMappings.Add("issue_resolved", "issue_resolved");

			adapter.TableMappings.Add(tableMapping);

			adapter.InsertCommand = new MySql.Data.MySqlClient.MySqlCommand();
			adapter.InsertCommand.CommandText = "INSERT INTO definition_results (scan_id, agent_id, definition_id, title, description, ref_source, ref_id, ref_url, affected_family, affected_platform, affected_product, result, issue_resolved) VALUES (?scan_id, ?agent_id, ?definition_id, ?title, ?description, ?ref_source, ?ref_id, ?ref_url, ?affected_family, ?affected_platform, ?affected_product, ?result, ?issue_resolved)";
			adapter.InsertCommand.Connection = this.Connection;
			adapter.InsertCommand.Parameters.Add ("agent_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("definition_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("title", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("description", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("ref_source", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("ref_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("ref_url", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("affected_family", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("affected_platform", MySql.Data.MySqlClient.MySqlDbType.VarChar);			
			adapter.InsertCommand.Parameters.Add ("affected_product", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("result", MySql.Data.MySqlClient.MySqlDbType.VarChar);
			adapter.InsertCommand.Parameters.Add ("scan_id", MySql.Data.MySqlClient.MySqlDbType.Int32);
			adapter.InsertCommand.Parameters.Add ("issue_resolved", MySql.Data.MySqlClient.MySqlDbType.Bit);
		}
		
		void InitCommandCollection() 
		{
            this.commandCollection = new MySql.Data.MySqlClient.MySqlCommand[2];
            
            this.commandCollection[0] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[0].Connection = this.Connection;
            this.commandCollection[0].CommandText = "SELECT * FROM definition_results";
            this.commandCollection[0].CommandType = System.Data.CommandType.Text;
            
            this.commandCollection[1] = new MySql.Data.MySqlClient.MySqlCommand();
            this.commandCollection[1].Connection = this.Connection;
            this.commandCollection[1].CommandText = "SELECT * FROM definition_results WHERE ref_id=?ref_id OR description like ?ref_id";
            this.commandCollection[1].CommandType = System.Data.CommandType.Text;
            this.commandCollection[1].Parameters.Add("ref_id", MySql.Data.MySqlClient.MySqlDbType.VarChar);
        }
		
		void InitConnection()
		{
			ConnectionStringSettings cs = ConfigurationManager.ConnectionStrings ["SussenWeb"];
			
			connection = new MySql.Data.MySqlClient.MySqlConnection();
			connection.ConnectionString = cs.ConnectionString;	
		}
		
		MySql.Data.MySqlClient.MySqlDataAdapter Adapter 
		{
            get {
            	if (this.adapter == null)
            		this.InitAdapter ();

                return this.adapter;
            }
        }
        
        MySql.Data.MySqlClient.MySqlConnection Connection 
        {
            get {     
            	if (this.connection == null)
            		this.InitConnection();

                return this.connection;
            }
            
            set {
            
                this.connection = value;
                if (this.Adapter.InsertCommand != null)
                    this.Adapter.InsertCommand.Connection = value;

                if (this.Adapter.DeleteCommand != null)
                    this.Adapter.DeleteCommand.Connection = value;
                
                if (this.Adapter.UpdateCommand != null)
                    this.Adapter.UpdateCommand.Connection = value;
                
                for (int i = 0; (i < this.CommandCollection.Length); i = (i + 1)) {
                    if ((this.CommandCollection[i] != null)) {
                        ((MySql.Data.MySqlClient.MySqlCommand)(this.CommandCollection[i])).Connection = value;
                    }
                }
            }
		}
		
		protected MySql.Data.MySqlClient.MySqlCommand[] CommandCollection 
		{
            get {
                if (this.commandCollection == null)
                    this.InitCommandCollection();

                return this.commandCollection;
            }
        }     
	}
}
