// 
// WebService.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Configuration;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml;
using ICSharpCode.SharpZipLib.Checksums;
using ICSharpCode.SharpZipLib.Zip;
using Sussen.Web.BusinessLogic;

[WebService(Namespace = "http://dev.mmgsecurity.com/projects/sussen")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class WebService : System.Web.Services.WebService 
{
    public WebService () 
    {
        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    [WebMethod]
    public string GetCompressionSupported () 
    {
    	return "zip";
    }

    [WebMethod]
    public int GetDefinitions (string compressionType, string familyName, byte[] systemInfo, out byte[] definitionData) 
    {
    	string xmlDirectory = WebConfigurationManager.AppSettings ["xmlDirectory"];
		string agentID = GetSystemInfoValue ("host_identification", systemInfo);
		if (agentID == null) {
			string familyFile = GetFamilyDefinitions (familyName);
			if (familyFile == null) {
				definitionData = null;
				return -1;
			}
			
			definitionData = EncodeDefinitionsFile (familyFile, compressionType);		
			return 0;
		}
    
        try
        {
            AgentBLL agentLogic = new AgentBLL();
            string definitions = agentLogic.GetAgentPolicy (agentID);            
            if (definitions == null) {
				string familyFile = GetFamilyDefinitions (familyName);
				if (familyFile == null) {			
					definitionData = null;
					return -2;
				}
			
				definitionData = EncodeDefinitionsFile (familyFile, compressionType);
				return 0;
            }
            
	    	string file = Path.Combine (xmlDirectory, definitions);
			definitionData = EncodeDefinitionsFile (file, compressionType);
			return 0;
			
        } catch {       
        	definitionData = null;
        	return -3;
        }        
    }

	[WebMethod]
    public string GetError (int errorCode) 
    {
    	switch (errorCode) {

		case 0:
			return "Operation successful";
			
		default:
			return "";
    	}
    }

    [WebMethod]
    public int GetSignatureXml (string compressionType, out byte[] data) 
    {
    	data = Encoding.UTF8.GetBytes ("");
    	return 1;
    }

    [WebMethod]
    public int RequestClientID (string compressionType, byte[] systemInfo, out byte[] clientData) 
    {    
		if (systemInfo.Length == 0) {
            string res = string.Format ("<host_identification>{0}</host_identification>", System.Guid.NewGuid().ToString());        
        	clientData = Encoding.UTF8.GetBytes (res);
        	return 0;
		}

		byte[] decodedSystemInfo = DecodeData (systemInfo, compressionType);
		if (decodedSystemInfo == null) {
			clientData = null;
			return -1;
		}
			
		string hostName = GetSystemInfoValue ("primary_host_name", decodedSystemInfo);
		if (hostName == null) {
			clientData = null;
			return -2;
		}
		
        try
        {
            AgentBLL agentLogic = new AgentBLL();
            string id = agentLogic.RegisterAgent(hostName);            
            string res = string.Format ("<host_identification>{0}</host_identification>", id);
        
        	clientData = Encoding.UTF8.GetBytes (res);
        	return 0;
        }
#if WINDOWS_BUILD
        catch (Exception e)
        {

            StringBuilder msg = new StringBuilder();
            msg.AppendFormat("\n\nAn error occured when {0} ({1}) was registering an agent with the system\n\n",
                HttpContext.Current.Request.UserHostName,
                HttpContext.Current.Request.UserHostAddress);

            msg.AppendFormat("Message: {0}\n\nStack trace:\n{1}", e.Message, e.ToString());


            EventLog log = new EventLog();
            log.Source = "Application";
            log.WriteEntry(msg.ToString(), EventLogEntryType.Error);
            log.Close();
        }
#else
		catch {}        
#endif
        
        clientData = null;
        return -3;
    }

    [WebMethod]
    public int ReturnResults(string compressionType, byte[] systemInfo, byte[] resultsData)
    {
        int retVal = -1;

        if (resultsData == null || resultsData.Length == 0 || compressionType == null || compressionType == string.Empty)
            return retVal;

        try
        {
            resultsData = DecodeData (resultsData, compressionType);
            string xmlData = Encoding.UTF8.GetString (resultsData);            
            string hostID = null;
            
            if (systemInfo != null) {
            	systemInfo = DecodeData (systemInfo, compressionType);
            	string systemData = Encoding.UTF8.GetString (systemInfo);            
            	hostID = GetHostIdentification (systemData);
            } else {
            	hostID = GetHostIdentification (xmlData);
            }
            
            if (hostID == null)
            	hostID = "unknown";

            ScanResultsBLL scanResultsLogic = new ScanResultsBLL();            
            scanResultsLogic.InsertScanResults(hostID, xmlData);
            
            return 0;
        }
#if WINDOWS_BUILD
        catch (Exception e)
        {

            StringBuilder msg = new StringBuilder();
            msg.AppendFormat("\n\nAn error occured when {0} ({1}) was uploading scan results\n\n",
                HttpContext.Current.Request.UserHostName,
                HttpContext.Current.Request.UserHostAddress);

            msg.AppendFormat("Message: {0}\n\nStack trace:\n{1}", e.Message, e.ToString());


            EventLog log = new EventLog();
            log.Source = "Application";
            log.WriteEntry(msg.ToString(), EventLogEntryType.Error);
            log.Close();	            
        }
#else
		catch {}        
#endif

        return retVal;
    }

    [WebMethod]
    public int ReturnSystemCharacteristics(string compressionType, byte[] systemInfo, byte[] scData)
    {
        int retVal = -1;

        if (scData == null || scData.Length == 0 || compressionType == null || compressionType == string.Empty)
            return retVal;

        try
        {
            scData = DecodeData (scData, compressionType);
            string scDoc = Encoding.UTF8.GetString (scData);
            
			string hostID = null;
            
            if (systemInfo != null) {
            	systemInfo = DecodeData (systemInfo, compressionType);
            	string systemData = Encoding.UTF8.GetString (systemInfo);            
            	hostID = GetHostIdentification (systemData);
            } else {
            	hostID = GetHostIdentification (scDoc);
            }
            
            if (hostID == null)
            	hostID = "unknown";
            	
            SystemCharacteristicsBLL scLogic = new SystemCharacteristicsBLL();            
            scLogic.InsertSystemCharacteristics(hostID, scDoc);
            
            return 0;
        }
#if WINDOWS_BUILD
        catch (Exception e)
        {

            StringBuilder msg = new StringBuilder();
            msg.AppendFormat("\n\nAn error occured when {0} ({1}) was uploading system characteristics\n\n",
                HttpContext.Current.Request.UserHostName,
                HttpContext.Current.Request.UserHostAddress);

            msg.AppendFormat("Message: {0}\n\nStack trace:\n{1}", e.Message, e.ToString());


            EventLog log = new EventLog();
            log.Source = "Application";
            log.WriteEntry(msg.ToString(), EventLogEntryType.Error);
            log.Close();	            
        }
#else
		catch {}        
#endif

        return retVal;
    }

	static byte[] DecodeData (byte[] encodedData, string compressionType)
	{
		if (encodedData == null || compressionType == null)
			return null;
						
		switch (compressionType) {
		
		case "none":
			return encodedData;
			
		case "zip":
			return DecodeAndUnzipFile (encodedData);
			
		default:
			return null;
		}
	}

	static byte[] EncodeDefinitionsFile (string fileName, string compressionType)
	{
		if (fileName == null || compressionType == null)
			return null;
		
		XmlDocument doc = new XmlDocument ();
		doc.Load (fileName);
	
		byte[] rawBytes = Encoding.UTF8.GetBytes (doc.InnerXml);
				
		switch (compressionType) {
		
		case "none":
			return rawBytes;	
			
		case "zip":
			return ZipAndEncodeFile (rawBytes, "definitions.xml");
			
		default:
			return null;
		}
	}

    static byte[] DecodeAndUnzipFile (byte[] rawBytes)
    {
        MemoryStream ms = new MemoryStream();
        ms.Write(rawBytes, 0, rawBytes.Length);
        ms.Seek(0, SeekOrigin.Begin);

        ZipInputStream zs = new ZipInputStream(ms);
        MemoryStream outMemoryStream = new MemoryStream();

        while (zs.GetNextEntry() != null)
        {
            int size = 4096;
            byte[] data = new byte[4096];

            do
            {
                size = zs.Read(data, 0, data.Length);
                outMemoryStream.Write(data, 0, size);
            } while (size > 0);
        }

        zs.Close();
        ms.Close();

        return outMemoryStream.ToArray();
    }
   
   	static string GetFamilyDefinitions (string familyName)
   	{
   		string xmlDirectory = WebConfigurationManager.AppSettings ["xmlDirectory"];
   		string fileName = string.Format ("{0}.definitions.xml", familyName);
   		
   		string fullPath = Path.Combine (xmlDirectory, fileName);
   		if (File.Exists (fullPath))
   			return fullPath;
   			
   		return null;
   	}
   
    static string GetHostIdentification (string xmlDocument)
    {
    	try {
    	
	    	XmlDocument doc = new XmlDocument();
	    	doc.LoadXml (xmlDocument);
	    	
			XmlNodeList nl = doc.GetElementsByTagName ("system_info");
			if (!(nl.Count > 0))
				return null;

			XmlElement si = (XmlElement) nl[0];
			nl = si.GetElementsByTagName ("host_identification");
			if (!(nl.Count > 0))
				return null;

			XmlElement hi = (XmlElement) nl[0];
			return hi.InnerText;
			
		} catch { }
		
		return null;
    }
   
    static string GetSystemInfoValue (string tagName, byte[] systemInfo)
    {
    	try {
    		
    		string xmlData = Encoding.UTF8.GetString (systemInfo);
    		
    		XmlDocument doc = new XmlDocument ();
	    	doc.LoadXml (xmlData);
	    	
	    	XmlNodeList nl = doc.GetElementsByTagName (tagName);
	    	if (!(nl.Count > 0))
	    		return null;
	    		
	    	XmlElement xe = (XmlElement) nl[0];
	    	return xe.InnerText;
	    	
	    } catch {
	    	return null;
	    }
    }
    
    static byte[] ZipAndEncodeFile (byte[] buf, string storageName)
	{
		MemoryStream ms = new MemoryStream(); 
		ZipOutputStream zs = new ZipOutputStream(ms);
		zs.SetLevel (6);

		Crc32 crc = new Crc32();

		ZipEntry ze = new ZipEntry (storageName);
		ze.DateTime = DateTime.Now;
		ze.Size = buf.Length;

		crc.Reset ();
		crc.Update(buf);

		ze.Crc = crc.Value;

		zs.PutNextEntry (ze);
		zs.Write(buf, 0, buf.Length);
		zs.Finish ();

		byte[] zipBytes = new byte [ms.Length];
		ms.Seek (0, SeekOrigin.Begin);
		ms.Read (zipBytes, 0, zipBytes.Length);

		zs.Close ();
		ms.Close ();

		return zipBytes;
	}
    
}

