CREATE TABLE agents(
	id varchar(150) NOT NULL PRIMARY KEY,
	name varchar(200) NOT NULL
);

CREATE TABLE agent_policy(
	agent_id varchar(150) NOT NULL,
	definitions varchar(255) NOT NULL,
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

CREATE TABLE scans(
	id int NOT NULL AUTO_INCREMENT PRIMARY KEY,
	agent_id varchar(150) NOT NULL,
	scan_date timestamp DEFAULT NOW(),
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

CREATE TABLE scan_results (
	scan_id int NOT NULL,
	agent_id varchar(150) NOT NULL,
	results longtext NOT NULL,
	FOREIGN KEY (scan_id) REFERENCES scans (id) ON DELETE CASCADE,
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

CREATE TABLE system_characteristics (
	scan_id int NOT NULL,
	agent_id varchar(150) NOT NULL,
	scxml longtext NOT NULL,
	FOREIGN KEY (scan_id) REFERENCES scans (id) ON DELETE CASCADE,
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

CREATE TABLE definition_results (
	id int NOT NULL AUTO_INCREMENT PRIMARY KEY,
	scan_id int NOT NULL,
	agent_id varchar(150) NOT NULL,
	definition_id varchar (255) NOT NULL,
	title varchar (255),
	description text,
	ref_source varchar (255),
	ref_id varchar (255),
	ref_url varchar (255),
	affected_family varchar (25),
	affected_platform varchar (255),
	affected_product varchar (255),
	result varchar (25) NOT NULL,
	issue_resolved boolean,
	FOREIGN KEY (scan_id) REFERENCES scans (id) ON DELETE CASCADE,
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

CREATE TABLE agent_status (
	agent_id varchar(150) NOT NULL,
	product_version varchar (10),
	last_scan timestamp,
	FOREIGN KEY (agent_id) REFERENCES agents (id) ON DELETE CASCADE
);

DELIMITER |

CREATE PROCEDURE CreateScan (agentID VARCHAR(150))
BEGIN
	INSERT INTO scans (agent_id) VALUES (agentID);
	SELECT LAST_INSERT_ID();
END|

CREATE PROCEDURE DeleteScan (id INT)
BEGIN
	DELETE FROM scans WHERE id=id;
	DELETE FROM scan_results WHERE scan_id=id;
	DELETE FROM definition_results WHERE scan_id=id;
END|

DELIMITER ;
