use master
go
CREATE DATABASE sussen
go
use sussen
go

CREATE TABLE agents(
	id varchar(200) NOT NULL ,
	name varchar(200) NOT NULL
)

CREATE TABLE agent_policy(
	agent_id varchar(150) NOT NULL,
	definitions varchar(255) NOT NULL
)

CREATE TABLE scans(
	id integer identity,
	agent_id varchar(150) NOT NULL,
	scan_date datetime NOT NULL DEFAULT (getdate())
)

CREATE TABLE scan_results (
	scan_id integer NOT NULL,
	agent_id varchar(150) NOT NULL,
	results varchar(max) NOT NULL
)

CREATE TABLE system_characteristics (
	scan_id integer NOT NULL,
	agent_id varchar(150) NOT NULL,
	scxml varchar(max) NOT NULL
)

CREATE TABLE definition_results (
	id integer identity,
	scan_id integer NOT NULL,
	agent_id varchar(150) NOT NULL,
	definition_id varchar (255) NOT NULL,
	title varchar (255),
	description varchar (max),
	ref_source varchar (255),
	ref_id varchar (255),
	ref_url varchar (255),
	affected_family varchar (25),
	affected_platform varchar (255),
	affected_product varchar (255),
	result varchar (25) NOT NULL,
	issue_resolved bit
)

CREATE TABLE agent_status (
	agent_id varchar(150) NOT NULL,
	product_version varchar (10),
	last_scan datetime
)

CREATE PROCEDURE RegisterAgent
	@id varchar(150),
	@name varchar (200)
AS
BEGIN
	SET NOCOUNT ON;
	INSERT INTO agents VALUES (@id, @name)
	INSERT INTO agent_status (agent_id) VALUES (@id)
END;

CREATE PROCEDURE DeleteAgent
	@id varchar(150)
AS
BEGIN
	SET NOCOUNT ON;
	DELETE FROM agents WHERE id=@id
	DELETE FROM agent_policy WHERE agent_id=@id
	DELETE FROM agent_status WHERE agent_id=@id
	DELETE FROM scans WHERE agent_id=@id
	DELETE FROM scan_results WHERE agent_id=@id
	DELETE FROM system_characteristics WHERE agent_id=@id
	DELETE FROM definition_results WHERE agent_id=@id
END;

CREATE PROCEDURE DeleteScan
	@id integer
AS
BEGIN
	SET NOCOUNT ON;
	DELETE FROM scans WHERE id=@id
	DELETE FROM scan_results WHERE scan_id=@id
	DELETE FROM definition_results WHERE scan_id=@id
END;

use sussen
go
exec sp_grantlogin 'MACHINENAME\ASPNET'
exec sp_grantdbaccess 'MACHINENAME\ASPNET'
GRANT EXECUTE on RegisterAgent to public
GRANT EXECUTE on DeleteAgent to public
GRANT EXECUTE on DeleteScan to public
go

